/*
 * Copyright (C) 2013, 2014
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import QtQuick 2.4
import Ubuntu.Components 1.3
import Ubuntu.Components.Popups 1.3

import "databasemodule_v2.js" as DB
import "util.js" as JS

Page {
    id: addWebSitePage

    objectName: "addWebSitePage"
    title: i18n.tr("Add Website")
    visible: false

    signal webSiteAdded()
    property string nakedUrl
    property bool oskVisible: false
    head.backAction: Action {
        iconName: "back"
        onTriggered: {
            Qt.inputMethod.hide()
            myTimer.delay(500, pageStack.pop)
        }
    }
    Component.onCompleted: {
        reloadPageContent ()
    }
   function reloadPageContent() {
        webTitle.text = ""
        webUrl.text = "http://"
     }

     function doPattern(url){
         var pattern = JS.getLocation(url).host
         if (JS.isSuperPattern()==true){
              if(pattern.split('.').length>2){
                  if(pattern.match(/^www\.(.*)/)){
                       pattern = pattern.replace(/^www\./g,'*.')
                  } else if(pattern.match(/^m\.(.*)/)){
                       pattern = pattern.replace(/^m\./g,'*.')
                  } else if(pattern.match(/^mobile\.(.*)/)){
                       pattern = pattern.replace(/^mobile\./g,'*.')
                  }
              }
              if(!pattern.match(/^\*\./)){
                   pattern="*."+pattern
              }
         }
         pattern = "https?://"+pattern+"/*"
         return pattern;
    }
    function doTitle(str){
        var title
        if (str.length>3){
            title = JS.capitalizeFirstLetter(str)
        } else {
            title = str.toUpperCase()
        }
        return title
    }
    function addWebSite(title, url, pattern, secondaryPatterns){
         var error=0
         if(title){
             title = title.trim()
             title = title.replace(/"/g, "'")
         }
         if(!title || title.length==0){
             var p = pattern.split(".")
             title = doTitle(p[1])
             if(p.length>3)title += " " + doTitle(p[2])
             console.log("title: "+ title)
         }
         var r = DB.addWebSite(title , url, pattern, secondaryPatterns)
         var m = i18n.tr("SUCCESS")
         if ( r.error == true) {
             error = 1
             if (r.exist==true) {
                 m = i18n.tr("ERROR: already exists")
                 console.log(title + " already exists!");
             } else {
                 m = i18n.tr("ERROR: too many websites")
                 console.log("failed in inserting " +title);
             }
         }
         util.popupMessage(m, title+"\n" + url+"\n" + pattern)
         if(error==0){
             util.updateSecondaryPatterns()
             webSiteAdded()
             reloadPageContent()
             log.statistics('addapp')
         }
         return error;
    }
    function insertWebSite(){
         var tmp = JS.dnToLowerCase(webUrl.text)
         if(tmp){
             webUrl.text = tmp
             nakedUrl = JS.getNakedUrl(webUrl.text)
             if(nakedUrl == webUrl.text){
                 addWebSite(webTitle.text, webUrl.text, doPattern(webUrl.text))
             } else {
                 PopupUtils.open(dialog)
             }
         } else {
             util.popupError(i18n.tr("ERROR: bad URL"), webUrl.text)
         }
    }
    Column
    {
        anchors{ left: parent.left; right: parent.right }
        spacing: units.gu(2)

        Row
        {
            anchors{ left: parent.left; right: parent.right; leftMargin: units.gu(2); rightMargin: units.gu(2) }
            spacing: units.gu(1)

            TextField
            {
                id: webTitle
                objectName: "webTitle"
                text: webSiteTitle
                placeholderText: i18n.tr("A Website title")
                width: parent.width - parent.spacing
                anchors.verticalCenter: parent.verticalCenter
                readOnly: false
                hasClearButton: true
            }
        }

        Row
        {
            anchors{ left: parent.left; right: parent.right; leftMargin: units.gu(2); rightMargin: units.gu(2) }
            spacing: units.gu(1)

            TextField
            {
                id: webUrl
                objectName: "webUrl"
                text: webSiteUrl
                placeholderText: i18n.tr("A Website url")
                width: parent.width - parent.spacing
                anchors.verticalCenter: parent.verticalCenter
                readOnly: false
                hasClearButton: true
            }
        }
        Row
        {
            anchors{ left: parent.left; right: parent.right; leftMargin: units.gu(2); rightMargin: units.gu(2) }
            spacing: units.gu(1)

            Text
            {
                id: infoTitle
                objectName: "infoTitle"
                text: i18n.tr("Procedure to configure websites:")
                anchors.verticalCenter: parent.verticalCenter
                wrapMode: Text.WordWrap
//                contentWidth : parent.width - parent.spacing
                width: parent.width - parent.spacing
                font.bold: true

            }
        }
        Row
        {
            anchors{ left: parent.left; right: parent.right; leftMargin: units.gu(2); rightMargin: units.gu(2) }
            spacing: units.gu(1)

            Text
            {
                id: info
                objectName: "info"
                text: 
i18n.tr("1) Download site configurations from Website Repository.")+"\n\n"
+ i18n.tr("2) Sites that are not available in Website Repository: fill their name and url on above fields and tap \"Insert\" button.")+"\n\n"
+ i18n.tr("3) Share your new configurations after verifying that they are working as expected. On \"Edit Website\" page tap \"Upload\" button to upload your sites to the Website Repository.")
                anchors.verticalCenter: parent.verticalCenter
                wrapMode: Text.WordWrap
//                contentWidth : parent.width - parent.spacing
                width: parent.width - parent.spacing

            }
        }
    }
    Rectangle {
        id: fakePanel

        color: "#fafafa"
        anchors {
            left: parent.left
            right: parent.right
            bottom: parent.bottom
        }
        height: units.gu(6)
        Item {
            anchors.fill: parent

            Button {
                id: downloadButton
                text: i18n.tr("Download")
                anchors {
                    verticalCenter: parent.verticalCenter
                    left: parent.left
                    leftMargin: units.gu(1)
                }
                onClicked: pageStack.push(downloadWebSitePage)
            }
            Button {
                id: actionsButton
                text: i18n.tr("Insert")
                anchors {
                    verticalCenter: parent.verticalCenter
                    right: parent.right
                    rightMargin: units.gu(1)
                }
                onClicked: {
                    if(!oskVisible){
                        insertWebSite()
                    }
                }
            }
        }
    } // Rectangle fakePanel
    onStateChanged:{
        if (actionsButton.pressed && oskVisible){
            util.click()
            insertWebSite()
        }
    }
    states: [
        State {
            name: "pressed"
            when: actionsButton.pressed
        }
    ]

    Connections {
        target: Qt.inputMethod
        onVisibleChanged: {
            oskVisible=!oskVisible
        }
    }
    /* -------------------------- dialogue ---------------------------- */
    Item {
        width: units.gu(80)
        height: units.gu(80)
        Component {
             id: dialog
             Dialog {
                 id: dialogue
                 title: i18n.tr("Insert")
                 Label {
                     text: i18n.tr("Use forced url only if recommended url does not work as you like:")+"\n\n"
                            +i18n.tr("recommended:")
                            +"\n    "+nakedUrl+"\n\n"
                            +i18n.tr("forced:")
                            +"\n    "+webUrl.text+"\n\n"
                            +i18n.tr("Your choice:")
                     wrapMode: Text.WordWrap
                 }

                 Button {
                     text: i18n.tr("recommended")
                     color: UbuntuColors.green
                     onClicked: {
                         addWebSite(webTitle.text, nakedUrl, doPattern(nakedUrl))
                         PopupUtils.close(dialogue)
                     }
                 }
                 Button {
                     text: i18n.tr("forced")
                     color: UbuntuColors.red
                     onClicked: {
                          addWebSite(webTitle.text, webUrl.text, doPattern(webUrl.text))
                          PopupUtils.close(dialogue)
                     }
                 }
            }
        }
    }
}
