import QtQuick 2.4
import "util.js" as JS
import "databasemodule_v2.js" as DB

Item {
    id: bookmarks

    property var titleRules: []
    property var objectName: "bookmarks"

    function init(rulesUpdated){
        if(!optionsKeeper.bmtrDownloaded() || optionsKeeper.bmtrDownloaded()<rulesUpdated){
            console.log("download a new bookmark title rules")
            util.doRequestCallback(resetTitleRules, 'GET','/BmtRules?order=myId')
        } else {
            titleRules=DB.loadBmtRules().rows
        }
    }

    function resetTitleRules(resp){
        var currentdate = new Date();
        DB.dropTable('bmt_rules')
        var rules = resp.results
        console.log('downloaded rules:'+rules.length)
        for (var i = 0; i < rules.length; i++){
            DB.addBmtRules(rules[i])
        }
        optionsKeeper.setBmtrDownloaded(currentdate.getTime());
        titleRules=DB.loadBmtRules().rows
    }

    function webSiteId(url, sites){
        var max=0;
        var id=undefined
        var bmUrl=util.getUrl(url)
        console.log("bookmark url: "+bmUrl)
        for (var i=0; i<sites.length; i++) {
            if (JS.doesMatch(bmUrl, sites[i].pattern) == true 
                  && sites[i].pattern.length>max) {
                max = sites[i].pattern.length
                id = sites[i].id
            }
        }
        return id
    }
    function bookmarkPosition(url){
        var sites = DB.loadWebSites().rows
        var id = webSiteId(url, sites)
        var position=undefined
        var positionId=undefined
        if(id){
            console.log('bookmarkPosition id: '+id)
            for (var i=0; i<sites.length; i++) {
                if(id==sites[i].id){
                    position = sites[i].position
                    positionId = JS.positionId(position)
                } else if(positionId!=undefined && positionId == JS.positionId(sites[i].position)){
                    position = sites[i].position
                } else if(positionId){
                    break
                }
            }
            if(position!=undefined){
                position = position + 0.01
            }
        } else {
            console.log('ERROR: app id not found!')
        }
        return position
    }
    function setHomePage(url){
        var id = webSiteId(url, DB.loadWebSites().rows)
        if (id) {
            console.log('sites['+id+'] homepage: '+url)
            DB.updateWebSiteHome(id, url.toString())
        }
    }
    function doTitle(bookmark){
        var href = util.getUrl(bookmark.url)
        var rule;
        for (var i=0; i<titleRules.length; i++) {
            var pattern = new RegExp(titleRules[i].pattern);
            if(href.match(pattern) && titleRules[i].type>0 &&
               (i<titleRules.length-1 || optionsKeeper.bmtrUseHttpReq())){
                console.log('match; '+href+', '+titleRules[i].pattern)
                rule=titleRules[i]
                break
            }
        }
        var dn = JS.getDomain(href)
        var title = href.replace(dn, "");
        var done = true
        if(rule && rule.type==1){
            var t = title
            var f 
            if(rule.useFragment){
                f = title.split('#')[1] 
            } else {
                title = title.split('#')[0]                  
            }
            if(!rule.useParams){
                title = title.split('?')[0] 
                if(f && title.indexOf('#') === -1){
                    title += '#'+f
                }                  
            }
            if(rule.regexSrc){
                console.log('rule.regexSrc: '+rule.regexSrc)
                var src = new RegExp(rule.regexSrc)
                if (rule.regexDest){
                    if(rule.regexDest=="''")rule.regexDest=""
                    console.log('rule.regexDest: '+rule.regexDest)
                    title = title.replace(src,rule.regexDest)
                } else if (rule.respTitle) {
                    var match = title.match(src)
                    console.log('match: '+match)
                    if(match) title = match[rule.respTitle]
                }
            }
            if(title.length<2){
                title = t
            }
            title = title.replace(/\//g," ")
            title = title.replace(/_/g," ")
            console.log('in: '+t)
            console.log('out: '+title)
        } else if(rule && rule.type==2){
            bookmark.title=decodeURIComponent(title)
            request4title(bookmark, rule)
            done=false
        } else if(rule && rule.type==3){
            bookmark.title=decodeURIComponent(title)
            var regex = new RegExp(rule.regexSrc);
            var match = title.match(regex)
            if(match){
                var lat =match[rule.lat]
                var lon =match[rule.long]
                var zoom = rule.zoom
                if(JS.isInt(rule.zoom)){
                    zoom=match[rule.zoom]
                    if(rule.respTitle)zoom = rule.respTitle*zoom
                }
                console.log('lat,long,zoom: '+lat+', '+lon+', '+zoom)
                reverseGeocode(lat, lon, zoom, bookmark)
                done=false
            } else {
                console.log('reverese geolocation: failed parsing url path')
                log.message(log.error, bookmark.url+', rule: '+JSON.stringify(rule))
            }
        } else {
            if(title.length<2){
                var parsed = JS.getLocation(href)
                title = parsed.host.split('.')[0]
            }
        }
        if(done){
            title = title.replace(/\+/g," ")
            bookmark.title = decodeURIComponent(addWayBackTitle(bookmark.url, title))
            insertBookmark(bookmark)
        }
    }
    function addWayBackTitle(url, title){
        var date=util.getWayBackDate(url)
        if(date.length>1){
            title="WB:"+date.substring(0, 6)+" "+title
        }
        return title
    }
//
// http://nominatim.openstreetmap.org/reverse?format=json&lat=54.9824031826&lon=9.2833114795&zoom=18&addressdetails=1
//
    function reverseGeocode(lat, lon, zoom, bookmark){
        var url="http://nominatim.openstreetmap.org/reverse?format=json&addressdetails=1&lat="
                +lat+"&lon="+lon+"&zoom="+Math.round(zoom)
        console.log('reverseGeocode: '+url)
        var http = new XMLHttpRequest()
        http.open('GET', url);
        http.setRequestHeader("User-Agent", util.webUA); 
        http.onreadystatechange = function() { // Call a function when the state changes.
            if (http.readyState == 4) {
                console.log("status: " + http.status)
                if (http.status == 200 || http.status == 201) {
                    console.log("resp: "+http.responseText)
                    var resp = JSON.parse(http.responseText);
                    bookmark.title = addWayBackTitle(bookmark.url, resp.display_name)
                } else {
                    console.log("ERROR: reverseGeocode bad response")
                }
                insertBookmark(bookmark)
            }
        }
        http.send(); 
    }

    function request4title(bookmark, rule){
        var http = new XMLHttpRequest()
        http.open('GET', bookmark.url);
        http.setRequestHeader("User-Agent", util.webUA); 
        http.onreadystatechange = function() { // Call a function when the state changes.
            if (http.readyState == 4) {
                console.log("status: " + http.status)
                if (http.status == 200 || http.status == 201) {
                    var resp = http.responseText;
//                    var regex = new RegExp("(.*)<title(.*)>(.*)(.|[\r\n])</title>(.*)");
                    var regex = new RegExp(rule.regexSrc);
                    var match = resp.match(regex)
                    if(match){
                        bookmark.title=addWayBackTitle(bookmark.url, parseHtmlEntities(match[rule.respTitle]))
                        log.message(log.info, bookmark.url+' => '+bookmark.title)
                    } else {
                        log.message(log.error, bookmark.url+', regex: '+rule.regexSrc)
                        console.log("ERROR: website title not resolved...")
                    }
                } else {
                    console.log("ERROR: request4title bad response")
                }
                insertBookmark(bookmark)
            }
        }
        http.send(); 
    }
    function truncate(title, len){
        var truncTitle = title
        if(title.length > len){
            var delim = " - "
            if (title.indexOf(" | ") >= 0){
                delim = " | "
            }
            var parts = title.split(delim)
            truncTitle = parts[0]
            if(parts.length>1){
                for (var i=1; i<parts.length; i++){
                    if(truncTitle.length+parts[i].length+3 < len){
                        truncTitle += delim + parts[i]
                    } else {
                        break
                    }
                }
            }
            if(truncTitle.length>len){
                truncTitle = JS.truncate(truncTitle, len)
            } 
        }
        return truncTitle 
    }

    function insertBookmark(bookmark){
        var title = truncate(bookmark.title, 32) 
        title = title.replace(/"/g, "'")
        console.log("insertBookmark: "+title)
        var r = DB.addBookmark(title, bookmark.url, bookmark.position)
        if ( r.error == true) {
            var m
            if (r.exist==true) {
                m = i18n.tr("ERROR: already exists")
                console.log(title + " already exists!");
            } else {
                m = i18n.tr("ERROR: too many websites")
                console.log("failed in inserting " +title);
            }
            util.popupMessage(m, title+"\n" + bookmark.url)
        }
    }

    function setBookmark(url){
        var pos = bookmarkPosition(url)
        if (pos) {
            console.log('url position: '+pos)
            var bookmark = {title: "",
                            url: url.toString(),
                            position: pos}
            doTitle(bookmark)
        } else {
             util.popupMessage('ERROR: app not found', url)
        }
    }
    function resetHomePage(url){
        var id = webSiteId(url, DB.loadWebSites().rows)
        if (id) {
            console.log('reset sites['+id+'] homepage')
            DB.resetWebSiteHome(id)
        }
    }
//
// http://stackoverflow.com/questions/7394748/whats-the-right-way-to-decode-a-string-that-has-special-html-entities-in-it
//
    function parseHtmlEntities(str) {
        return str.replace(/&#([0-9]{1,3});/gi, function(match, numStr) {
            var num = parseInt(numStr, 10); // read num as normal number
            return String.fromCharCode(num);
        });
    }
}

