import QtQuick 2.4
import Ubuntu.Components 1.3
import Ubuntu.Components.ListItems 1.3 as ListItem

import "util.js" as JS

ListItem.Empty {
//ListItem {    // new listItem
    id: categoryComponent

    height: columnContent.height
    showDivider : true  // old ListItem

    property bool isExpanded: false
    property string categoryTitle
    property string categoryId
    property bool appFilter
    property bool isSelected: false

    property bool checkDone: false

//    onClicked: isExpanded = !isExpanded ;  

    onStateChanged:reload()

    Behavior on height {
        NumberAnimation { duration: 150; easing.type: Easing.OutQuad }
    }

    function reload(){
        if (isExpanded){
            reloadWebSites()
        } else {
            checkDone=false
        }
    }

    function reloadWebSites (){
        categoryWebSiteModel.clear()
        var subCategories = downloadWebSitePage.subCategories
        var uri = '/WebSite/?where='
        var filter = '{"category":"'+categoryTitle+'","duplicate":null'
        if(subCategories.length>0){
             filter = filter +',"subCategory":{"$in":'+subCategories+'}'
        }
        if(appFilter){
             filter = filter +',"tags":"'+JS.getAppTag()+'"'
        }
        filter = filter + '}'
        uri = uri+encodeURIComponent(filter)+'&order=name';
        util.doRequestCallback(fillCategoryWebSiteModel, "GET", uri)
    }
 
    function fillCategoryWebSiteModel(resp){
        var ws = {}
        var max = -1
        for (var i = 0; i < resp.results.length; i++){
            var safe=util.websiteSafeCode(resp.results[i].safe) 
            categoryWebSiteModel.append(
            {
                "id": resp.results[i].objectId
               ,"title": resp.results[i].name
               ,"url": resp.results[i].url
               ,"pattern": resp.results[i].pattern
               ,"secondaryPatterns": resp.results[i].secondaryPatterns ? JSON.stringify(resp.results[i].secondaryPatterns) : ""
               ,"safe": safe
               ,"mobi": resp.results[i].mobi
               ,"mobiTst": resp.results[i].mobi===0 || resp.results[i].mobi===1 ? 1 : 0 
               ,"updatedAt": resp.results[i].updatedAt
               ,"rank": resp.results[i].rank && util.showRank() ? resp.results[i].rank : ""
            })

            if(!resp.results[i].state || (resp.results[i].state && resp.results[i].state!=2)){
                var ago = JS.daysAgo(resp.results[i].updatedAt)
//                if((ago>util.parseConf.urlcheckMinInt-1 && ago > max) || 
//                    resp.results[i].safe==null || resp.results[i].safe<0 ){
                if(ago>0 && ago > max){ 
                    max = ago
                    ws.updatedAt = resp.results[i].updatedAt
                    ws.safeCode = safe.color
                    ws.objectId = resp.results[i].objectId
                    ws.url = resp.results[i].url
                    ws.rank = resp.results[i].rank
                    console.log("ws.url: "+ws.url+" /"+ago)
/* mrn 14/2/2016: check only one website!
// urlcheckExpire obsolate! */
/*                  if(ago>util.parseConf.urlcheckExpire-1){
                        util.urlCheck(ws)
                        done=true 
                    }
*/
                }
            }
        }
        if(ws.url && !checkDone){
            checkDone=true
            console.log("check ws.url: "+ws.url)
            util.urlCheck(ws)
        }
    }

    Column{
        id: columnContent
        anchors{ left: parent.left; right: parent.right }

        /*
          category item
        */
        Item
        {
            id: categoryHeader
            anchors{ left: parent.left; right: parent.right }
            height: units.gu(5)
           /*
              category header for showing name
           */
            ListItem.Base{
                id: headerCategory
                objectName: "headerCategory"
                removable: false
                height: units.gu(5)
                onClicked: {
                    isExpanded = !isExpanded
                }
            }

            Row{
                id: rowCategoryContent
                anchors
                {
                    top: parent.top; bottom: parent.bottom; left: parent.left;
                    leftMargin: units.gu(1); topMargin: units.gu(0.7); bottomMargin: units.gu(1);
                }
                spacing: units.gu(2)
                Label{
                    id: labelCategoryTitle
                    objectName: "labelCategoryTitle"
                    anchors.verticalCenter: parent.verticalCenter
                    text: categoryTitle
                }
            }

            Image{
                id: imgArrow
                anchors
                {
                    right: parent.right; top: parent.top; bottom: parent.bottom;
                    topMargin: units.gu(1.5); bottomMargin: units.gu(1.5); rightMargin: units.gu(2)
                }
                fillMode: Image.PreserveAspectFit
                source: Qt.resolvedUrl("../icons/go-to.svg")
                smooth: true

                Behavior on rotation { UbuntuNumberAnimation{} }

                states: [
                    State {
                        name: "expanded"
                        when: categoryComponent.isExpanded

                        PropertyChanges
                        {
                            target: imgArrow
                            rotation: 90
                        }
                    }
                ]
            }

        }

        /*
          categoryWebSites listview
        */
        ListView
        {
            id: categoryWebSiteList
            anchors{ left: parent.left; right: parent.right }
            height: 0
            opacity: 0
            interactive: false
            clip: true

            property int transitionDuration: 250

            Behavior on opacity {
                PropertyAnimation { duration: 150; easing.type: Easing.OutQuad }
            }

            model: VisualDataModel {
                id: visualModel
                model: ListModel {
                    id: categoryWebSiteModel
                }

                CategoryWebSiteComponent
                {
                    id: categoryWebSiteItem
                    name: model.title
                    url: model.url
                    pattern: model.pattern
                    secondaryPatterns: model.secondaryPatterns 
                    safe: model.safe
                    mobi: model.mobi
                    mobiTst: model.mobiTst
                    updatedAt: model.updatedAt
                    rank: model.rank
                    categoryWebSiteId: model.id
                    categoryId: categoryComponent.categoryId
                    rowHeight: 6
                    width: categoryList.width
                    height: units.gu(rowHeight)
                }
            }
        }
    }
    states: [
        State {
            name: "expanded"
            when: isExpanded
            PropertyChanges
            {
                target: categoryWebSiteList
                height: categoryWebSiteList.contentItem.childrenRect.height
                opacity: 1
            }
        },

        State {
            name: ""
//            when: isExpanded
            PropertyChanges
            {
                target: categoryWebSiteList
                height: 0
                opacity: 0
            }
        }
    ]

    Connections {
        target: util
        onWebsiteUpdated: reload()
    }

}
