import QtQuick 2.4
import Ubuntu.Components 1.3
import Ubuntu.Components.Popups 1.3
import "util.js" as JS

Item
{
   id: categoryWebSiteComponent

    property string name
    property string url
    property string pattern
    property string secondaryPatterns
    property string rank
    property var safe
    property string categoryWebSiteId
    property string categoryId
    property var mobi
    property var mobiTst
    property var updatedAt
    property var rowHeight

    function checkUrl(){
        var ws = {}
        ws.url = url
        ws.objectId = categoryWebSiteId
        ws.updatedAt = updatedAt
        ws.safeCode = safe.color
        ws.rank = rank
        util.urlCheck(ws)
    }
    function download(){
        checkUrl()
        console.log('secondaryPatterns: '+secondaryPatterns)
        var error=addWebSitePage.addWebSite(name, url, pattern, secondaryPatterns)
        if(error==0){
            var body = '{"downloads":{"__op":"Increment","amount":1}'
            if(JS.getAppTag() && JS.isDynamicAppTag() && categoryId){
                body = body +', "tags":{"__op":"AddUnique", "objects":["'+JS.getAppTag()+'"]}'
            }
            body = body + '}'
            util.doRequest("PUT", '/WebSite/'+categoryWebSiteId, body)
            if(JS.getAppTag() && JS.isDynamicAppTag() && categoryId){
                body = '{"tags":{"__op":"AddUnique","objects":["'+JS.getAppTag()+'"]}}'
                util.doRequest("PUT", '/Category/'+categoryId, body)
            }
        }
    }
    
    ListItem {
        height: units.gu(rowHeight)
        selectMode : false
        Captions {
            anchors {
                left: parent.left
                leftMargin: units.gu(2)
            }
            title.text: name
            subtitle.text: url
            subtitle.color: "darkGrey"
        }
        Label {
            anchors {
                right: parent.right
                rightMargin: units.gu(5)
            }
            text: rank
            fontSize: "xx-small"
        }
        Button {
            anchors {
                right: parent.right
                rightMargin: units.gu(1)
            }
            width: url ? units.gu(3) : 0
            height: units.gu(3)
            iconSource: safe.icon ? Qt.resolvedUrl("../icons/"+safe.icon+".svg") : ""
            color: safe.color ? safe.color : ""
            onClicked: {
                util.urlCheckView(url)
                checkUrl()
            }
        }
        Icon {
            anchors {
                right: parent.right
                bottom: parent.bottom
                rightMargin: units.gu(1)
            }
            width: mobiTst ? units.gu(3) : units.gu(0)
            height: units.gu(3)
            name: mobi ? "starred" : "non-starred"
        }
        trailingActions: url ? trailingAct : noAct
    }
    ListItemActions {
        id: trailingAct
        actions: [
            Action {
                name: categoryWebSiteComponent.name
                iconName: "save"
                onTriggered: {
                    download()
                }
            },
            Action {
                iconName: "external-link"
                onTriggered: {
                    Qt.openUrlExternally(url)
                    checkUrl()
                }
            },
            Action {
                iconName: "info"
                onTriggered: {
                    util.urlRankView(url)
                    checkUrl()
                }
            },
            Action {
                iconName: "history"
                onTriggered: {
                    util.wayBackView(url)
                    checkUrl()
                }
            }
        ] 
    }
    ListItemActions {
        id: noAct
        actions: [] 
    }
}


