import QtQuick 2.4
import Ubuntu.Components 1.3
import Ubuntu.Components.Popups 1.3
// import Ubuntu.Layouts 1.0

import "util.js" as JS

Page {
    id: downloadWebSitePage
    objectName: "downloadWebSitePage"

    title: i18n.tr("Repository")
    flickable: content
//    tools: toolbar

    property int subCategory
    property string subCategories
    property bool filtered : true
    property bool tagEnabled : JS.appTagFilter() 
    property bool oskVisible : false

    /* -------------------------- states ---------------------------- */
    state: "default"
    states: [
        PageHeadState {
            name: "default"
            head: downloadWebSitePage.head
            actions: [searchAction, filterAction, appTagAction, refreshAction]
        },
        PageHeadState {
            id: headerState
            name: "search"
            head: downloadWebSitePage.head
            actions: [filterSearchAction]
            backAction: Action {
                id: leaveSearchAction
                text: "back"
                iconName: "back"
                onTriggered: {
                    loadCategories()
                    downloadWebSitePage.state = "default"
                }
            }
            contents: TextField {
                id: searchText
                width: units.gu(30)
                placeholderText: i18n.tr("search...")
            }
       }
    ]

    Component.onCompleted: {
        loadCategories ()
    }

    function fillSubCategoryModel(unique){
        subCategoryModel.clear()
        var c=JS.getCategoryIds()
        var a = []
        for (var i = 0; i < c.length; i++){
            if(JS.contains(unique,c[i]))a.push(c[i])
        }
        console.log('fillSubCategoryModel>>>'+c)
        var len=a.length
        if (c.length>1 && c[0]==0){
            subCategoryModel.append(
            {
               "id": c[0]
               ,"name": JS.getCategoryName(c[0])
            })
            combo.text  = JS.getCategoryName(c[0])
            subCategory = c[0]
            len++
        } else { 
            combo.text= JS.getCategoryName(a[0])
            subCategory = a[0]
        }
        var comboLen = tagEnabled || downloadWebSitePage.state == "search" ? 0 : len
        combo.expandedHeight= units.gu(JS.getComboBoxExpandedHeight(comboLen))
        for (var i = 0; i < a.length; i++){
            subCategoryModel.append(
            {
               "id": a[i]
              ,"name": JS.getCategoryName(a[i])
            })
         }                                           
    }

    function toggleFilter(){
        filtered = !filtered
        loadCategories()
    }

    function loadCategoriesUri(){
        var f = ""
        subCategories = ""
        if(subCategory!=0){
            subCategories = '['+subCategory+']'
            f='{"subCategory":'+subCategory
        } else if (JS.useAllCategories() == false) {
            subCategories = JSON.stringify(JS.getCategoryIds())
            f='{"subCategory":{"$in":'+subCategories+'}'
        } 
        if(filtered && settingsPage.domainFilter.length>5){
            if(f){
                f=f+','
            } else {
                f='{'
            }
            f=f+'"name":{"$in":'+settingsPage.domainFilter+'}'
        }
        if(tagEnabled){
            if(f){
                f=f+','
            } else {
                f='{'
            }
            f = f +'"tags":"'+JS.getAppTag()+'"'
        }
        if(f){
            f='&where='+encodeURIComponent(f+'}')
        }
        console.log("f:"+f)
        var uri = '/Category/?order=name'+f;
        return uri
    }

    function loadCategories(){
        if(util.gTLD[0]){ 
            util.doRequestCallback(fillCategoryModels, "GET", loadCategoriesUri())
        }
    }

    function reloadCategories ()
    {
        util.doRequestCallback(fillCategoryModel, "GET", loadCategoriesUri())
    }

    function fillCategoryModel(resp){
        categoryModel.clear()
        var unique=[]
        for (var i = 0; i < resp.results.length; i++){
            unique = JS.arrayUnique(unique.concat(resp.results[i].subCategory));
            categoryModel.append(
            {
                "id": resp.results[i].objectId
               ,"title": resp.results[i].name
            })
        }
        return unique
    }
    function fillCategoryModels(resp){
        var unique=fillCategoryModel(resp)
        fillSubCategoryModel(unique)
    }
    function searchBadResult(result){
        webSiteModel.append(
        {
            "id": ""
           ,"title": result
           ,"url": ""
           ,"pattern": ""
           ,"secondaryPatterns": ""
           ,"safe": {color:"darkGrey",icon:undefined}
           ,"rank": "N/A"
           ,"mobi": 0
           ,"mobiTst": 0 
        })
    }
    function fillWebSiteModel(resp){
        console.log('search found: '+resp.results.length)
        if(resp.results.length==0){
            searchBadResult(i18n.tr("Not found"))
        } else {
            for (var i = 0; i < resp.results.length; i++){
                var safe=util.websiteSafeCode(resp.results[i].safe) 
                webSiteModel.append( 
                {
                    "id": resp.results[i].objectId
                   ,"title": resp.results[i].name
                   ,"url": resp.results[i].url
                   ,"pattern": resp.results[i].pattern
                   ,"secondaryPatterns": resp.results[i].secondaryPatterns ? JSON.stringify(resp.results[i].secondaryPatterns) : ""
                   ,"safe": safe
                   ,"rank": resp.results[i].rank
                   ,"mobi": resp.results[i].mobi
                   ,"mobiTst": resp.results[i].mobi===0 || resp.results[i].mobi===1 ? 1 : 0 
                   ,"updatedAt": resp.results[i].updatedAt
                })
            }
        }
    }
    function doSearch(){
        webSiteModel.clear()
        var search
        if(searchText.text)search=searchText.text.trim()
        if(!search || search.length<3){
            if(search.length>0){
                searchBadResult(i18n.tr("Search string is too short!"))
            } else {
                searchText.text=""
            }
        } else {
            var f='"duplicate":null'
            if(filtered && settingsPage.domainFilter.length>5){
                f+=',"category":{"$in":'+settingsPage.domainFilter+'}'
            }
            var t = '(.*)'+search.replace(/\./g, "\\.")+'(.*)'
            var cond=JSON.stringify({$regex:t,$options:"i"})
            f+=',"$or":[{"name":'+cond+'},{"url":'+cond+'}]'
            var uri = '/WebSite/?where={'+encodeURIComponent(f)+'}';
            util.doRequestCallback(fillWebSiteModel, "GET", uri)
        }
    }
    /*
      main content
    */
    Flickable {
        id: content
        anchors {
            left: parent.left
            right: parent.right
            top: parent.top
            bottom: parent.bottom
       }
        contentHeight: contentItem.childrenRect.height
 
        Column{
            anchors{ left: parent.left; right: parent.right }

            ComboButton {
                id: combo
                collapsedHeight : downloadWebSitePage.state == "search" ? 0 : units.gu(5)
                onClicked: expanded = false
                anchors {
                    left: parent.left
                    right: parent.right
                    leftMargin: units.gu(1); rightMargin: units.gu(2)
                }
                ListView {
                    model: ListModel {
                        id: subCategoryModel
                    }
                    delegate: ListItem { 
                        Label {
                            text: name
                        }
                        onClicked: {
                             console.log(id+": "+model.name)
                             combo.text = model.name;
                             subCategory = model.id;
                             combo.expanded = false;
                             reloadCategories()
                        }
                    }
                }
            }
            /*
              Category listview
            */
            ListView {
                id: categoryList
                anchors{ left: parent.left; right: parent.right }
                height: downloadWebSitePage.state == "default" ? contentItem.childrenRect.height : -1
                interactive: false
                model: VisualDataModel {
                    id: visualModel
                    model: ListModel {
                        id: categoryModel
                    }

                    CategoryComponent{
                        id: categoryItem
                        categoryTitle: model.title
                        categoryId: model.id
                        appFilter: tagEnabled 
                    }
                }
            } // listview

            /*
              Search listview
            */
            ListView {
                id: searchList
                anchors{ left: parent.left; right: parent.right }
                height: downloadWebSitePage.state == "default" ? -1 : contentItem.childrenRect.height
                interactive: false
                model: VisualDataModel {
                    id: visualSearchModel
                    model: ListModel {
                        id: webSiteModel
                    }

                    CategoryWebSiteComponent{
                        id: categoryWebSiteItem
                        name: model.title
                        url: model.url
                        pattern: model.pattern
                        secondaryPatterns: model.secondaryPatterns 
                        safe: model.safe
                        rank: model.rank
                        mobi: model.mobi
                        mobiTst: model.mobiTst
                        updatedAt: model.updatedAt
                        categoryWebSiteId: model.id
                        rowHeight: 6
                        width: categoryList.width
                        height: units.gu(rowHeight)
                    }
                }
            } // listview
        } // column
    } // flickable

/*-------------------------- Actions ---------------------------- */
    Action {
        id: refreshAction
        text:  i18n.tr("Refresh")
        iconSource: Qt.resolvedUrl("../icons/reload.svg")
        onTriggered: reloadCategories()
    }
    Action {
        id: filterAction
        text:  i18n.tr("Domain filter")
        iconSource: filtered ?  Qt.resolvedUrl("../icons/filter.svg") : Qt.resolvedUrl("../icons/no-filter.svg")
        onTriggered: toggleFilter()
    }
    Action {
        id: filterSearchAction
        text:  i18n.tr("Domain filter")
        iconSource: filtered ?  Qt.resolvedUrl("../icons/filter.svg") : Qt.resolvedUrl("../icons/no-filter.svg")
        onTriggered: {
            filtered = !filtered
            if(searchText.text.trim().length>2)doSearch()
        }
    }

    Action {
        id: appTagAction
        visible: JS.getAppTag() ? true : false 
        text:  i18n.tr("App filter")
        iconName: tagEnabled ? "ok" : "close"
        onTriggered: {
            tagEnabled = !tagEnabled
            if(tagEnabled)subCategory=0
            loadCategories()
        }
    }
    Action {
        id: searchAction
        iconName: "search"
        onTriggered: {
            downloadWebSitePage.state = "search"
            combo.expandedHeight=0
            if(searchText.text.trim().length>2)doSearch()
        }
    }

/*-------------------------- Connections ---------------------------- */
    Connections {
        target: editWebSite

        onWebSiteUploaded:{
            console.log("onWebSiteUploaded")
            loadCategories()
        }
    }
    Connections {
        target: settingsPage

        onFilterChanged:{
            console.log("onFilterChanged")
            loadCategories()
        }
    }
    Connections {
        target: Qt.inputMethod
        onVisibleChanged: {
            oskVisible=!oskVisible
            if(!oskVisible){
                 doSearch()
            }
        }
    }
}
