import QtQuick 2.4
import Ubuntu.Components 1.3
import Ubuntu.Components.Popups 1.3
// import Ubuntu.Layouts 1.0

import "databasemodule_v2.js" as DB
import "util.js" as JS

Page {
    id: editWebSite
    objectName: "editwebsitepage"
    title: i18n.tr("Edit Website")
//    flickable: content

    property int modelIndex
    property int webSiteId
    property string webSiteTitle: ""
    property string webSiteUrl: ""
    property string webSitePattern: ""
    property string secondaryPatterns: ""
    property string webSiteHome: ""
    property var parseData
    property int comboBoxItems
    property var comboColor
    property int category
    property string modifyInfo
    property string nakedUrl
    property string bookMarkAction: "..."
    property bool bookMarkActionReady: false
    property bool isBookmark: false
    property bool oskVisible: false

    signal webSiteEdited(int id, string label)
    signal webSiteUploaded()

    head.actions: [resetAction]

    Component.onCompleted: {
        reloadSubCategories ()
    }

    function reloadSubCategories ()
    {
        categoryModel.clear()
        var c = JS.getCategories()
        comboBoxItems=0;
        for (var i=0; i < c.length; i++){
            if(c[i].id!=0){
                categoryModel.append(
                {
                    "id": c[i].id
                    ,"name": c[i].name
                })
                comboBoxItems++
            }
        }
        combo.text = i18n.tr("undefined")
        category = 0
        if(comboBoxItems==1){
            combo.text=c[0].name
            category = c[0].id
        }
        combo.expandedHeight= units.gu(JS.getComboBoxExpandedHeight(comboBoxItems))        
    }
    function init(webId, modelId){
        console.log('editwebsitepage: '+webId+', '+modelId)
        modelIndex = modelId
        setValues(webId)
        if(comboBoxItems>1){
            combo.text = i18n.tr("undefined")
            category = 0
        }
        comboColor = UbuntuColors.lightGrey
        combo.dropdownColor = comboColor
        bookMarkActionReady = false
        if(isBookmark){
            bookMarkAction= "-"
        } else {
            bookMarkAction= "..."
            parseRest.searchConfiguration(analyzeSearchResult, webSiteUrl)
        }
    }

    function analyzeSearchResult(result){
        parseData = result
        comboColor = UbuntuColors.red
        bookMarkAction = i18n.tr("Upload")
        if(parseData){
             util.urlCheck(parseData)
             console.log("subCategory:"+parseData.subCategory)
             if(comboBoxItems>1){
                combo.text = JS.getCategoryName(parseData.subCategory)
                category = JS.getCategoryId(parseData.subCategory)
             }
             if(parseData.name==webSiteTitle && 
                parseData.pattern==webSitePattern &&
                !parseData.duplicate && 
                category > 0){
                 comboColor = UbuntuColors.green
                 bookMarkAction= i18n.tr("Align")
             } else {
                 comboColor = '#E77E5A'
                 if(parseData.duplicate || (parseData.state && parseData.state==1)){
                    bookMarkAction= i18n.tr("Download")
                 } else {
                    bookMarkAction= i18n.tr("Align")
                 }
             }
             if((parseData.secondaryPatterns && secondaryPatterns!=JSON.stringify(parseData.secondaryPatterns))
                 || (secondaryPatterns && !parseData.secondaryPatterns)){
                 var p = JSON.stringify(parseData.secondaryPatterns)
                 DB.updateWebSiteSecondaryPatterns(webSiteId, p)
                 util.updateSecondaryPatterns()
                 util.popupMessage("UPDATED: secondary URL patterns", p)
             }
        }
        combo.dropdownColor = comboColor
        bookMarkActionReady = true
    }

    function isModified(){
        modifyInfo = ""
        if(parseData.name != webSiteTitle){
             modifyInfo=i18n.tr("Name:")+"\n   "+parseData.name+"\n   "+webSiteTitle+"\n"
        }
        if(parseData.duplicate && parseData.url != webSiteUrl){
             modifyInfo = modifyInfo
                 + i18n.tr("Url:")+"\n   "+parseData.url+"\n   "+webSiteUrl+"\n"
        }
        if(parseData.pattern != webSitePattern){
             modifyInfo = modifyInfo 
                 + i18n.tr("Pattern:")+"\n   "+parseData.pattern+"\n   "+webSitePattern+"\n"
        }
        if(parseData.subCategory != category){
             modifyInfo = modifyInfo 
                 + i18n.tr("Category:")+"\n   "+JS.getCategoryName(parseData.subCategory)
                 +          "\n   "+JS.getCategoryName(category)+"\n"
        }
        if(modifyInfo.length>0)return true;
        return false;
    }
    function setValues(webId)
    {
        console.log("webSiteId: "+webId)
        webSiteId = webId ;
        var webSite = DB.getWebSite(webId)
        isBookmark=false
        if(JS.bookmarkId(webSite.rows[0].position))isBookmark=true
        var insertTime=""
        if(isBookmark && webSite.rows[0].insertTime)insertTime=webSite.rows[0].insertTime
        webSiteTitle = webSite.rows[0].title;
        webSiteUrl = webSite.rows[0].source;
        webSitePattern = webSite.rows[0].pattern ? webSite.rows[0].pattern : insertTime;
        secondaryPatterns = webSite.rows[0].secondaryPatterns ? webSite.rows[0].secondaryPatterns : "";
        webSiteHome = webSite.rows[0].home;
        webTitle.text=webSite.rows[0].title;
        webUrl.text=webSiteUrl;
        webPattern.text=webSitePattern;
    }

    function updateWebSite(myTitle, myUrl, myPattern){
        var m
        var error=0
        myTitle = myTitle.replace(/"/g, "'")
        var r= DB.updateWebSiteByUser(webSiteId, myTitle.trim(), myUrl, myPattern)
        if ( r.error == true && r.exist==true) {
            error = 1
            m = i18n.tr("ERROR: website already exists")
        } else if ( r.rowsAffected != 1) {
            m = i18n.tr("ERROR: updated %1 websites").arg(r.rowsAffected)
            error = 1
        }
        if( error == 0 ){
            if(myUrl != webSiteUrl){
                DB.resetWebSiteHome(webSiteId)
            }
            webSiteEdited(modelIndex, myTitle.trim())
            pageStack.pop()
        } else {
            util.popupError(m, myTitle+"\n" + myUrl+"\n" + myPattern)
        }
    }
    function preUpdateWebSite(){
        var m
        var error=0
        if(isBookmark){
            updateWebSite(webTitle.text.trim(), webUrl.text, "")
        } else if (JS.doesMatch(webUrl.text, webPattern.text)==true) {
            var p = webPattern.text.split(':\/\/');
            if( p.length<2 || p[0].length<1 || p[1].length<6) {
                error = 1
                m = i18n.tr("ERROR: invalid pattern!")
            } else {
                var dnl = JS.dnToLowerCase(webUrl.text)
                if(!dnl){
                    error = 1
                    m = i18n.tr("ERROR: bad URL")
                } else if(dnl == webUrl.text){ 
                    nakedUrl = JS.getNakedUrl(webUrl.text)
                    if(nakedUrl == webUrl.text){
                        updateWebSite(webTitle.text.trim(), webUrl.text, webPattern.text)
                    } else {
                        PopupUtils.open(updateDialog)
                    }
                } else {
                    error = 1
                    m = i18n.tr("ERROR: upper case char(s) in domain name")
                }
            }
        } else {
            error = 1
            m = i18n.tr("ERROR: website URL does not match with pattern")
        }
        if( error != 0 ){
            util.popupError(m, webTitle.text+"\n" + webUrl.text+"\n" + webPattern.text)
        }
    }

/*
    Flickable {
        id: content
        anchors {
            left: parent.left
            right: parent.right
            top: parent.top
            bottom: fakePanel.top
        }
        contentHeight: contentItem.childrenRect.height
*/
        Column
        {
            anchors{ left: parent.left; right: parent.right }
            spacing: units.gu(2)

            ComboButton {
                id: combo
                onClicked: {
                     expanded = !expanded
                     if(expanded){
                          combo.dropdownColor = '#DEDBD8'
                     } else {
                          combo.dropdownColor = comboColor
                     }
                }
                anchors {
                    left: parent.left
                    right: parent.right
                    leftMargin: units.gu(2); rightMargin: units.gu(3)
                }
                ListView {
                    model: ListModel {
                        id: categoryModel
                    }
                    delegate: ListItem {
                        Label {
                            text: name
                        }
                        onClicked: {
                            console.log(id+": "+model.name)
                            combo.text = model.name;
                            category = model.id;
                            combo.expanded = false;
                        }
                    }
                }
            }

            Row
            {
                anchors{ left: parent.left; right: parent.right; leftMargin: units.gu(2); rightMargin: units.gu(2) }
                spacing: units.gu(1)

                TextField
                {
                    id: webTitle
                    objectName: "webTitle"
                    text: webSiteTitle
                    width: parent.width  - parent.spacing
                    anchors.verticalCenter: parent.verticalCenter
                    readOnly: false
                    hasClearButton: false
                }
            }

            Row
            {
                anchors{ left: parent.left; right: parent.right; leftMargin: units.gu(2); rightMargin: units.gu(2) }
                spacing: units.gu(1)

                TextField
                {
                    id: webUrl
                    objectName: "webUrl"
                    text: webSiteUrl
                    width: parent.width  - parent.spacing
                    anchors.verticalCenter: parent.verticalCenter
                    readOnly: isBookmark ? true : false
                    hasClearButton: false
                }
            }
            Row
            {
                anchors{ left: parent.left; right: parent.right; leftMargin: units.gu(2); rightMargin: units.gu(2) }
                spacing: units.gu(1)

                TextField
                {
                    id: webPattern
                    objectName: "webPattern"
                    text: webSitePattern
                    width: parent.width - parent.spacing
                    anchors.verticalCenter: parent.verticalCenter
                    readOnly: isBookmark ? true : false
                    hasClearButton: false
                }
            }
            Row
            {
                anchors{ left: parent.left; right: parent.right; leftMargin: units.gu(2); rightMargin: units.gu(2) }
                spacing: units.gu(1)

                Text
                {
                    id: colorTitle
                    objectName: "colorTitle"
                    text: i18n.tr("ComboBox color and desired actions:")
                    anchors.verticalCenter: parent.verticalCenter
                    wrapMode: Text.WordWrap
                    width: parent.width - parent.spacing
                    font.bold: true
                }
            }
            Row
            {
                anchors{ left: parent.left; right: parent.right; leftMargin: units.gu(2); rightMargin: units.gu(2) }
                spacing: units.gu(1)

                Text
                {
                    id: colorInfo
                    objectName: "colorInfo"
                    text: 
    i18n.tr("Green: Site is on the Website Repository with identical configuration.")+"\n\n"
    + i18n.tr("Orange: Site is on the Website Repository with different configuration. Tap \"Align\" button and from dialog popup choose your action.")+"\n\n"
    + i18n.tr("Red: Site not found from the Website Repository. Choose site category from ComboBox and tap \"Upload\" button")
                    anchors.verticalCenter: parent.verticalCenter
                    wrapMode: Text.WordWrap
                    width: parent.width - parent.spacing

                }
            }
        }
//    }


    Rectangle {
        id: fakePanel
        color: "#fafafa"
        anchors {
            left: parent.left
            right: parent.right
            bottom: parent.bottom
        }
        height: units.gu(6)
        Item {
            anchors.fill: parent

            Button {
                id: uploadButton
                text: bookMarkAction
                anchors {
                    verticalCenter: parent.verticalCenter
                    left: parent.left
                    leftMargin: units.gu(1)
                }
                onClicked: {
                    if(bookMarkActionReady){
                        if(category>0){
                            setValues(webSiteId)
                            parseRest.init(parseData, webSiteTitle, webSiteUrl, webSitePattern, category)
                            if(!parseData){
                                nakedUrl = JS.getNakedUrl(webUrl.text)
                                if(nakedUrl == webUrl.text){
                                    parseRest.uploadWebSite()
                                    pageStack.pop()
                                } else {
                                    PopupUtils.open(uploadDialog)
                                }
                            } else if (parseData.duplicate){
                                isModified()
                                PopupUtils.open(downloadDialog)
                            } else if (isModified() == true){
                                if(!parseData.state || (parseData.state && parseData.state!=1)){
                                    PopupUtils.open(dialog)
                                } else {
                                    PopupUtils.open(downloadDialog)
                                }
                            } else {
                                pageStack.pop()
                            }
                        } else {
                            util.popupError(i18n.tr("ERROR: invalid category"), i18n.tr("undefined"))
                        }
                    }
                }
            }
            Button {
                id: actionsButton
                text: i18n.tr("Update")
                anchors {
                    verticalCenter: parent.verticalCenter
                    right: parent.right
                    rightMargin: units.gu(1)
                }
                onClicked: {
                    if(!oskVisible){
                        preUpdateWebSite()
                    }
                }
            }
        }
    } // Rectangle fakePanel
    onStateChanged:{
        if (actionsButton.pressed && oskVisible){
            util.click()
            preUpdateWebSite()
        }
    }
    states: [
        State {
            name: "pressed"
            when: actionsButton.pressed
        }
    ]

    Connections {
        target: Qt.inputMethod
        onVisibleChanged: {
            oskVisible=!oskVisible
        }
    }
/*-------------------------- Actions ---------------------------- */
    Action {
        id: resetAction
        visible: webSiteHome===webSiteUrl ? false : true 
        text:  i18n.tr("Reset")
        iconSource: Qt.resolvedUrl("../icons/reset.svg")
        onTriggered: PopupUtils.open(resetDialog)
    }

    /* -------------------------- dialogue ---------------------------- */
    Item {
        width: units.gu(80)
        height: units.gu(80)
        Component {
             id: dialog
             Dialog {
                 id: dialogue
                 title: i18n.tr("Upload")
                 Label {
                     text: i18n.tr("Upload/download website configuration?")+"\n"
                           + modifyInfo
                     wrapMode: Text.WordWrap
                 }
                 Button {
                     text: i18n.tr("upload")
                     color: UbuntuColors.green
                     onClicked: {
                         parseRest.uploadWebSite()
                         pageStack.pop()
                         PopupUtils.close(dialogue)
                     }
                 }
                 Button {
                     text: i18n.tr("download")
                     color: UbuntuColors.green
                     onClicked: {
                         updateWebSite(parseData.name, parseData.url, parseData.pattern)
                         PopupUtils.close(dialogue)
                     }
                 }
                 Button {
                     text: i18n.tr("cancel")
                     onClicked: PopupUtils.close(dialogue)
                 }
            }
        }
    }
    Item {
        width: units.gu(80)
        height: units.gu(80)
        Component {
             id: resetDialog
             Dialog {
                 id: dialogue
                 title: i18n.tr("Reset website homepage")
                 Label {
                     text: webSiteHome+"\n"
                           +"\t=>\n"
                           + webSiteUrl
                     wrapMode: Text.WordWrap
                 }
                 Button {
                     text: i18n.tr("reset")
                     color: UbuntuColors.green
                     onClicked: {
                         log.statistics('resetwh')
                         DB.resetWebSiteHome(webSiteId)
                         setValues(webSiteId)
                         webSiteEdited(null, null)
                         PopupUtils.close(dialogue)
                     }
                 }
                 Button {
                     text: i18n.tr("cancel")
                     onClicked: PopupUtils.close(dialogue)
                 }
            }
        }
    }
    Item {
        width: units.gu(80)
        height: units.gu(80)
        Component {
             id: downloadDialog
             Dialog {
                 id: dialogue
                 title: i18n.tr("Download")
                 Label {
                     text: i18n.tr("Download a website new configuration?\n"+
                                modifyInfo)
                     wrapMode: Text.WordWrap
                 }
                 Button {
                     text: i18n.tr("download")
                     color: UbuntuColors.green
                     onClicked: {
                         updateWebSite(parseData.name, parseData.url, parseData.pattern)
                         PopupUtils.close(dialogue)
                     }
                 }
                 Button {
                     text: i18n.tr("cancel")
                     onClicked: PopupUtils.close(dialogue)
                 }
            }
        }
    }
    Item {
        width: units.gu(80)
        height: units.gu(80)
        Component {
             id: updateDialog
             Dialog {
                 id: dialogue
                 title: i18n.tr("Update")
                 Label {

// TRANSLATORS: dialog window for a polite suggestion to use url: http://example.com/ (recommended/nakedUrl) 
// TRANSLATORS: instead of http://example.com/index.html or  
// TRANSLATORS: http://example.com/index.html?param=1&day=2 (forced/webUrl.text) 
// TRANSLATORS:
// TRANSLATORS: text: i18n.tr("Use forced url only if recommended url does not work as you like:")+"\n\n"
// TRANSLATORS:      +i18n.tr("recommended:")
// TRANSLATORS:      +"\n    "+nakedUrl+"\n\n"
// TRANSLATORS:      +i18n.tr("forced:")
// TRANSLATORS:      +"\n    "+webUrl.text+"\n\n"
                     text: i18n.tr("Use forced url only if recommended url does not work as you like:")+"\n\n"
                            +i18n.tr("recommended:")
                            +"\n    "+nakedUrl+"\n\n"
                            +i18n.tr("forced:")
                            +"\n    "+webUrl.text+"\n\n"
                            +i18n.tr("Your choice:")
                     wrapMode: Text.WordWrap
                 }

                 Button {
                     text: i18n.tr("recommended")
                     color: UbuntuColors.green
                     onClicked: {
                         updateWebSite(webTitle.text.trim(), nakedUrl, webPattern.text)
                         PopupUtils.close(dialogue)
                     }
                 }
                 Button {
                     text: i18n.tr("forced")
                     color: UbuntuColors.red
                     onClicked: {
                         updateWebSite(webTitle.text.trim(), webUrl.text, webPattern.text)
                         PopupUtils.close(dialogue)
                     }
                 }
            }
        }
    }
    Item {
        width: units.gu(80)
        height: units.gu(80)
        Component {
             id: uploadDialog
             Dialog {
                 id: dialogue
                 title: i18n.tr("Warning")
                 Label {
// TRANSLATORS: text: i18n.tr("Upload the url")
// TRANSLATORS:       + "\n    "+webUrl.text+"\n\n"
// TRANSLATORS:       + i18n.tr("only if url")
// TRANSLATORS:       + "\n    "+nakedUrl+"\n\n"
// TRANSLATORS:       + i18n.tr("does not work as you like.")
                     text: i18n.tr("Upload the url")
                         + "\n    "+webUrl.text+"\n\n"
                         + i18n.tr("only if url")
                         + "\n    "+nakedUrl+"\n\n"
                         + i18n.tr("does not work as you like.")
                     wrapMode: Text.WordWrap
                 }

                 Button {
                     text: i18n.tr("upload")
                     color: UbuntuColors.green
                     onClicked: {
                         parseRest.uploadWebSite()
                         pageStack.pop()
                         PopupUtils.close(dialogue)
                     }
                 }
                 Button {
                     text: i18n.tr("cancel")
                     onClicked: {
                         PopupUtils.close(dialogue)
                     }
                 }
            }
        }
    }

    Connections {
        target: parseRest

        onParseReady:{
           console.log("onParseReady")
           webSiteUploaded()
        }
    }
}
