import QtQuick 2.4
import Ubuntu.Components 1.3
import "util.js" as JS

ListItem {
    id: faqComponent

    height: columnContent.height

    property bool isExpanded: false
    property string faqTitle
    property string faqAnswer
    property int modelIndex
    property bool isSelected: false

    onStateChanged:{
        if (isExpanded){
            reloadFaq()
        }
    }

    Behavior on height {
        NumberAnimation { duration: 150; easing.type: Easing.OutQuad }
    }

    function reloadFaq (){
        detailModel.clear()
        detailModel.append(
        {
            "id": modelIndex
            ,"question": faqTitle
            ,"answer": faqAnswer
        })

    }

    Column{
        id: columnContent
        anchors{ left: parent.left; right: parent.right }

        /*
          faq item
        */
        Item
        {
            id: headerFaq
            anchors{ left: parent.left; right: parent.right }
            height: units.gu(5)

            MouseArea {
                id: mouse
                enabled: true
                anchors { fill: parent; margins: units.gu(-1) }

                onClicked: {
                    faqComponent.isExpanded = !faqComponent.isExpanded
                }
            }


            Row{
                id: rowFaqContent
                anchors
                {
                    top: parent.top; bottom: parent.bottom; left: parent.left;
                    leftMargin: units.gu(1); topMargin: units.gu(0.7); bottomMargin: units.gu(1);
                }
                spacing: units.gu(2)


//                Text
                Label
                {
                    id: labelFaqTitle
                    objectName: "labelFaqTitle"
                    text: faqTitle
                    anchors.verticalCenter: parent.verticalCenter
//                    wrapMode: Text.WordWrap
                    width: parent.width - parent.spacing
//                    font.bold: true
                }
            }

            Image{
                id: imgArrow
                anchors
                {
                    right: parent.right; top: parent.top; bottom: parent.bottom;
                    topMargin: units.gu(1.5); bottomMargin: units.gu(1.5); rightMargin: units.gu(2)
                }
                fillMode: Image.PreserveAspectFit
                source: Qt.resolvedUrl("../icons/go-to.svg")
                smooth: true

                Behavior on rotation { UbuntuNumberAnimation{} }

                states: [
                    State {
                        name: "expanded"
                        when: faqComponent.isExpanded

                        PropertyChanges
                        {
                            target: imgArrow
                            rotation: 90
                        }
                    }
                ]
            }

        }

        /*
          details listview
        */
        ListView
        {
            id: detailList
            anchors{ left: parent.left; right: parent.right }
            height: 0
            opacity: 0
            interactive: false
            clip: true

            property int transitionDuration: 250

            Behavior on opacity {
                PropertyAnimation { duration: 150; easing.type: Easing.OutQuad }
            }

            model: VisualDataModel {
                id: visualModel
                model: ListModel {
                    id: detailModel
                }
//                Text
                Label
                {
                    id: answer
                    text: JS.wordWrap(model.answer,45,'\n',3)       // WORK AROUND
                    anchors.verticalCenter: parent.verticalCenter
//                    wrapMode: Text.Wrap                             // does not wrap!
                    width: parent.width - parent.spacing
                }
            }
        }
    }



    states: [
        State {
            name: "expanded"
            when: isExpanded
            PropertyChanges
            {
                target: detailList
                height: detailList.contentItem.childrenRect.height
                opacity: 1
            }
        },

        State {
            name: ""
            PropertyChanges
            {
                target: detailList
                height: 0
                opacity: 0
            }
        }
    ]
}
