import QtQuick 2.4
import Ubuntu.Components 1.3
import Ubuntu.Components.Popups 1.3
import "faq.js" as FAQ

Page {
    id: infoPage
    objectName: "infoPage"

    title: i18n.tr("Info")
    flickable: content

    Component.onCompleted: {
        console.log("infoPage")
        loadFAQ ()
    }


    function loadFAQ ()
    {
        faqModel.clear()
        var f = FAQ.loadAll()
        for (var i=0; i < f.length; i++){
           faqModel.append(
            {
                "id": f[i].id
                ,"question": f[i].question
                ,"answer": f[i].answer
            })
        }
    }


    /*
      main content
    */
    Flickable {
        id: content
        anchors {
            left: parent.left
            right: parent.right
            top: parent.top
            bottom: fakePanel.top
        }
        contentHeight: contentItem.childrenRect.height
 
        Column{
            anchors{ left: parent.left; right: parent.right }

            /*
              Faq listview
            */
            ListView {
                id: faqList
                anchors{ left: parent.left; right: parent.right }
                height: contentItem.childrenRect.height
                interactive: false
                currentIndex: -1

                property int faqSelectedIndex: -1

                signal collapseAllItem()

                function collapseAll(){
                    collapseAllItem()
                }

                function scrollView(itemy) {
                    console.log("itemy: ", itemy - content.contentY)
                    if ((itemy - content.contentY) < units.gu(16)) {
                        if (!content.atYBeginning)
                            content.contentY -= 3
                    }
                    else if ((itemy - content.contentY) > ( content.height - units.gu(16))) {
                        if (!content.atYEnd)
                            content.contentY += 3
                    }
                }

                displaced: Transition {
                    NumberAnimation { properties: "x,y"; easing.type: Easing.OutQuad }
                }

                model: VisualDataModel {
                    id: visualModel
                    model: ListModel {
                        id: faqModel
                    }
                    delegate: MouseArea {
                        id: delegateRoot

                        property int visualIndex: VisualDataModel.itemsIndex
                        property alias isExpanded: faqItem.isExpanded
                        property bool isSelected: visualIndex == faqList.faqSelectedIndex

                        anchors{ left: parent.left; right: parent.right }
                        height: faqItem.height

                        FaqComponent{
                            id: faqItem
                            faqTitle: model.question
                            faqAnswer: model.answer
                            modelIndex: model.id

                            states: [
                                State {
                                    name: ""
                                    PropertyChanges {
                                        target: faqItem
                                        isSelected: false
                                    }
                                },
                                State {
                                    name: "selected"
                                    when: delegateRoot.isSelected
                                    PropertyChanges {
                                        target: faqItem
                                        isSelected: true
                                    }
                                }
                            ]
                        }

                        Connections{
                            id: connFaqList
                            target: faqList

                            onCollapseAllItem:
                            {
                                delegateRoot.isExpanded = false
                            }
                        }
                    }
                }
            } // listview
        } // column
    } // flickable
    Rectangle {
        id: fakePanel

        color: "#fafafa"
        anchors {
            left: parent.left
            right: parent.right
            bottom: parent.bottom
        }
        height: units.gu(6)
        Item {
            anchors.fill: parent

            Button {
                id: supportButton
                text: i18n.tr("Support")
                anchors {
                    verticalCenter: parent.verticalCenter
                    right: parent.right
                    rightMargin: units.gu(1)
                }
                onClicked: {
                    Qt.openUrlExternally('https://launchpad.net/my-webapp-group')
                    log.statistics('support')
                }
            }
            Button {
                id: actionsButton
                text: i18n.tr("Version")
                anchors {
                    verticalCenter: parent.verticalCenter
                    left: parent.left
                    leftMargin: units.gu(1)
                }
                onClicked: myVersion.show()
            }
        }
    } // Rectangle fakePanel
}
