import QtQuick 2.4
import "util.js" as JS

Item {
    id: log

    property var app 
    property var revno
    property var parseConf
    property var error : 0
    property var warn : 1
    property var info : 2
    property var actions : {"newuser":0, "olduser":0, "resetdb":1, "info":1}

    function message(level, message){
        if(parseConf==null)return
        var file = (new Error).stack.split("\n")[1].split("/").pop();
        console.log(file+': '+message)
        if(level<parseConf.logLevel){
            console.log('log2Parse...')
            var msg = {"app":app,"version":revno,"file":file,"level":level,"message":message}
            util.doRequestCallback(logIt
                                  ,'GET'
                                  ,'/AppLog/?count=1&limit=1&order=updatedAt'
                                  ,'', JSON.stringify(msg))
        }
    }
    function logIt(resp, msg){
        if(resp.count<parseConf.logMaxRcrds){
            util.doRequest('POST', '/AppLog', msg)
        } else {
            util.doRequest('PUT', '/AppLog/'+resp.results[0].objectId, msg)
        }
    }
    function getMonth(d){
        var yyyy = d.getFullYear().toString();
        var mm = (d.getMonth()+1).toString(); // getMonth() is zero-based
        return yyyy + "." + (mm[1]?mm:"0"+mm[0]); // padding
    }
//
// http://stackoverflow.com/questions/6117814/get-week-of-year-in-javascript-like-in-php
//
    function getWeek(d){
        var yyyy = d.getFullYear().toString();
        d.setHours(0,0,0);
        d.setDate(d.getDate()+4-(d.getDay()||7));
        var ww = Math.ceil((((d-new Date(d.getFullYear(),0,1))/8.64e7)+1)/7).toString();
        return yyyy + (ww[1]?ww:"0"+ww[0]);
    }
//
// http://stackoverflow.com/questions/3066586/get-string-in-yyyymmdd-format-from-js-date-object
//
    function getDate(d){
        var yyyy = d.getFullYear().toString();
        var mm = (d.getMonth()+1).toString(); // getMonth() is zero-based
        var dd  = d.getDate().toString();
        return yyyy + (mm[1]?mm:"0"+mm[0]) + (dd[1]?dd:"0"+dd[0]); // padding
    }
    function getDateHour(d){
        var date = getDate(d)
        var hh  = d.getHours().toString();
        return date + (hh[1]?hh:"0"+hh[0]); // padding
    }
    function statistics(act){
        if(parseConf==null)return
        console.log('statistics: '+act)
        if(parseConf.statLevel > 2 ||
           (actions[act]!=null && actions[act] < parseConf.statLevel)){
            console.log('stat2Parse...')
            var loc = [null,null]
            var d = new Date();
            var dh 
            if(parseConf.statGranularity==3){
                dh = getMonth(d)
            } else if(parseConf.statGranularity==2){
                dh = getWeek(d)
            } else {
                dh = getDate(d)
            }
            if(act!='olduser' ||  !parseConf.statUsers || optionsKeeper.appLaunch()==null 
               || dh > optionsKeeper.appLaunch() || dh.length != optionsKeeper.appLaunch().length){
                if(parseConf.statUsers && (act=='olduser' || act=='newuser')){
                    optionsKeeper.setAppLaunch(dh)
                    loc = JS.splitLocale(Qt.locale().name)
                }
                if(parseConf.statGranularity==0){
                    dh = getDateHour(d)
                }
                var msg = '{"dateHour":'+dh+', "app":"'+app+'", "version":'+revno
                         +', "language":"'+loc[0]+'", "country":"'+loc[1]+'", "action":"'+act+'"'
                util.doRequestCallback(statIt, 'GET', '/AppStat/?where='+msg+'}', '', msg+',"count":1}')
            }
        }
    }
    function statIt(resp, msg){
        if(resp.results && resp.results.length>0){
            var body = '{"count":{"__op":"Increment","amount":1}}'
            util.doRequest('PUT', '/AppStat/'+resp.results[0].objectId, body)
        } else {
            util.doRequest('POST', '/AppStat', msg)
        }
    }
}
