import QtQuick 2.4
import Ubuntu.Components 1.3
import Ubuntu.Web 0.2
import "UCSComponents"
// import Ubuntu.Content 1.1
import Ubuntu.Components.Popups 1.3
import "actions" as Actions
import "."
import ".."
import "../config.js" as Conf
import "databasemodule_v2.js" as DB
import "util.js" as JS

MainView {
    id: mainView
    property var objectName: "mainView"

    applicationName: "mynewsapps.mattirn"
    automaticOrientation : true 
    anchorToKeyboard: true
    readonly property real latestDBVersion: 1.7

    property var bTitle : Conf.buttonTitle
    property var bWidth : Conf.buttonWidth
    property var navButtons : true
    property var navButtonSize : 0
    property var historyButton : true
    property bool historyEnabled : false
    property var wayBackUrl:""
    property var wayPresentUrl:""
    property var bottomActionUp: true
    property var appStartup: true
    property string myUA: Conf.webappUA ? Conf.webappUA : "Mozilla/5.0 (Linux; Android 5.0; Nexus 5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/38.0.2125.102 Mobile Safari/537.36"
    property var currentWebview:  webview.currentWebview
    property bool popupWindowOverlay: webview.popupWindowOverlay
    signal webSitesReset()
    signal settingsUpdated()

    Component.onCompleted: {
        util.initParseConf()
        // Update database if the db scheme is old.
        // optionsKeeper.setDbVersion(1.5)
        var dbv = parseFloat(optionsKeeper.dbVersion()+0.001)
        console.log("db version: " , dbv)
        if (dbv < latestDBVersion) {
            log.message(log.info, 'DB upgrade: '+dbv+' -> '+latestDBVersion)
            log.statistics('dbupgrade')
//
// put version change here because if db has been reset version has not been updated 
// and the upgrades here will fail... for ever. Or till next reset which has been corrected now.
//
            optionsKeeper.setDbVersion(latestDBVersion)
            if(dbv < 1.1){
                DB.updateDatabase(1.1)
                console.log ("set default domain filter values...")
                optionsKeeper.setgTLDfilter(true)
                optionsKeeper.setLangFilter(true)                    
            } 
            if(dbv < 1.2){
                DB.updateDatabase(1.2)
                console.log ("set default value for popup window...")
                optionsKeeper.setAllowPopup(true)
            }
            if(dbv < 1.3){
                DB.updateDatabase(1.3)
                var sites=DB.loadWebSites().rows
                for (var i=0; i<sites.length; i++) {
                    DB.resetWebSiteHome(sites[i].id)
                }
            }
            if(dbv < 1.4){
                DB.updateDatabase(1.4)
                optionsKeeper.setMyPatterns("")
                optionsKeeper.setUseDownloadedPatterns(true)
                var sites=DB.loadWebSites().rows
                for (var i=0; i<sites.length; i++) {
                    DB.updateWebSitePosition(sites[i].id, i)
                }
            }
            if(dbv < 1.5){
                DB.updateDatabase(1.5)
                console.log ("set default value for bookmark title request...")
                optionsKeeper.setBmtrUseHttpReq(true)
            }
            if(dbv < 1.7){
                DB.updateDatabase(1.7)
                console.log ("set default value for history button...")
                optionsKeeper.setHistoryButton(true)
                console.log ("set default value for navigation button size...")
                var nb = optionsKeeper.navButtons()
                if(nb){
                   optionsKeeper.setNavButtonSize(1)
                } else {
                   optionsKeeper.setNavButtonSize(3)
                }
            }
            settingsUpdated()
        }
        //  have to execute after db upgrade block
        util.updateSecondaryPatterns()

        // Filling DB with default data 
        // or
        // doing db alignment checks & optimations
        // DB.dropTable()

        var result = DB.webSitesCount()

        if (result.count < 1) {
            log.statistics('newuser')
            console.log('getDefWebSites...')
            parseRest.getDefWebSites(resetWebSites)
        } else {
            log.statistics('olduser')
            var sites=DB.loadWebSites().rows
            log.statistics('websites:'+sites.length)
            var curPID=undefined,curBID=undefined;
            var compPID=0,compBID=0;
            for (var i=0; i<sites.length; i++) {
                //
                // compress website positions
                //
                if(curPID==undefined)curPID=JS.positionId(sites[i].position)
                curBID = JS.bookmarkId(sites[i].position)
                if(JS.positionId(sites[i].position)>curPID){
                    curPID=JS.positionId(sites[i].position)
                    compPID=compPID+1
                }
                if(curBID){
                    compBID=compBID+1
                } else {
                    compBID=0
                }
                if(curPID!=compPID || (curBID && curBID!=compBID)){
                    var pos = compPID + compBID/100
                    console.log('new pos: '+pos)
                    DB.updateWebSitePosition(sites[i].id, pos)
                }
                if(curBID)continue
                //
                // might happen that after pattern modification website homepage does
                // not match any more with a new pattern
                //
                if (JS.doesMatch(sites[i].home, sites[i].pattern) == false) {
                    console.log("reset home")
                    DB.resetWebSiteHome(sites[i].id)
                }
            }
            getAppConfs()
        }
        appStartup=false
    }

    function getAppConfs(){
        var appHome = optionsKeeper.appHome()
        if(!appHome || !currentWebview.isValid(appHome)){
            var sites = DB.loadWebSites()
            optionsKeeper.setAppHome(sites.rows[0].home)
        }
        navButtons = optionsKeeper.navButtons()
        navButtonSize = optionsKeeper.navButtonSize()+2
        historyButton = optionsKeeper.historyButton()
        if(!historyButton)historyEnabled=false
    }

    function resetWebSites(defWebSites){
        optionsKeeper.setAppHome(undefined)
        optionsKeeper.setDbVersion(latestDBVersion)
        DB.dropTable('web_site')
        var loc = JS.splitLocale(util.getLocale())[0]
        for (var i = 0; i < defWebSites.length; i++){
            DB.addWebSite(defWebSites[i].name , 
                          defWebSites[i].url, 
                          defWebSites[i].pattern, JSON.stringify(defWebSites[i].secondaryPatterns))
            if(Conf.appHome && defWebSites[i].url.match('(.*)'+Conf.appHome+'(.*)')){
                optionsKeeper.setAppHome(defWebSites[i].url)
            }
            if(JS.contains(Conf.mlSites, defWebSites[i].url)){
                if(loc){
                    var href = defWebSites[i].url.replace(/:\/\//g, ':\/\/'+loc+'\.')
                    util.pingUrl(setHomePage, href)
                }
            }
        }
        getAppConfs()
        util.updateSecondaryPatterns()
        webSitesReset()
    }
    function canShowIt (enabled){
        if(navButtons && !popupWindowOverlay && nav.visible)return enabled
        return ''
    }
    function showHistoryButton(){
        if(historyButton && !historyEnabled) return true
        return false 
    }
    function switchMenu(){
        bottomAct.actions = bottomAct.my_actions()
        bottomAct.caller = invisible;
        bottomAct.show();
    }   
    PageStack {
        id: pageStack
        Component.onCompleted: {
             push(webPage)
        }
    }
        /* -------------------------- Pages & Tabs ---------------------------- */
       // ******************************** web ***********************///////////////


    Page {
	    id: webPage
	    visible: false
        property var objectName: "webPage"
     	anchors {
	        fill: parent
	        bottom: parent.bottom
	    }
	    width: parent.width
	    height: parent.height
        header: Rectangle {
            width: units.gu(0)
            height: units.gu(0)
        }

	    WebContext {
	        id: webcontext
	        userAgent: myUA
        }

        Item {
            id: webviewContainer
            anchors.fill: parent

            WebappContainerWebview {
                id: webview
                objectName: "webview"
                localUserAgentOverride: mainView.myUA
                url: optionsKeeper.appHome()
                anchors {
                    fill: parent
        	        bottom: parent.bottom
                }
                height: parent.height
  	            width: parent.width
            }
        }

        //--------------------------------------------
        //
        ThinProgressBar {
            webview: currentWebview
            anchors {
                left: parent.left
                right: parent.right
                top: parent.top
            }
        }
        //--------------------------------------------
        //
        RadialBottomEdge {
            id: nav
            visible: true
// sometimes fails try with sizes...
//            visible: mainView.popupWindowOverlay ? false : true 
            hintSize: mainView.popupWindowOverlay ? 0 : units.gu(8)
            actionButtonSize: mainView.popupWindowOverlay ? 0 : units.gu(7)
            bottomAction: bottomAct
            actions: [
                RadialAction {
	                id: home
                    iconName: showHistoryButton() ? "history" : "home"
	                onTriggered: {
                        if(showHistoryButton()){
                            historyEnabled=true
                            wayPresentUrl = currentWebview.url.toString()
                            currentWebview.url = util.getWayBackUrl(currentWebview.url)
                            log.statistics('wayback')
                        } else {
                            if(historyEnabled && wayPresentUrl.length>0){
                                currentWebview.url=wayPresentUrl
                            } else {
	                            currentWebview.url = optionsKeeper.appHome()
                            }
                            wayPresentUrl=""
                            historyEnabled=false
                        }
                        wayBackUrl=""
	                }
                },
                RadialAction {
                    id: forward
                    enabled: navButtons ? true : currentWebview.canGoForward
                    iconName: navButtons ?  "stock_website" : "go-next"
                    onTriggered: {
                        if(navButtons){
                            pageStack.push(downloadWebSitePage)
                        } else {
                            currentWebview.goForward()
                        }
                    }
                },   
                RadialAction {
                    id: save
                    iconName: "bookmark"
                    onTriggered: PopupUtils.open(dialog)
	            },
                RadialAction {
                    id: reload
                    iconName: "reload"
                    onTriggered: {
                        currentWebview.reload()
                    }
                },
	            RadialAction {
                    id: settings
                    iconName: "settings"
                    onTriggered: {
// reloadWebSites() is here because cannot be done before db ubgrade!
                        webSiteManagement.reloadWebSites()
                        pageStack.push(webSiteManagement)
                    }
                },
                RadialAction {
                    id: back
                    enabled: navButtons ? true : currentWebview.canGoBack
                    iconName: navButtons ? "add" : "go-previous"
                    onTriggered: {
                        if(navButtons){
                            pageStack.push(addWebSitePage)
                        } else {
                            currentWebview.goBack()
                        }
                    }
                }
            ]
        }
        //--------------------------------------------
        //
        // http://askubuntu.com/questions/450142/actionselectionpopover-populate-dynamically
        //
        // first button click ok... then get following error in every menu item:
        // file:///usr/lib/x86_64-linux-gnu/qt5/qml/Ubuntu/Components/Popups/
        //    ActionSelectionPopover.qml:144: TypeError: Cannot read property of null
        //    144:                width: parent.width
        // in any case it does not present any real fault. 
        //
        ActionSelectionPopover {
            id: bottomAct
            function my_actions(){
                var actions = []
                var sites=DB.loadWebSites().rows
                var msec = (new Date).getTime()
                var actStart = 0;
                var actEnd = sites.length;
                if(sites.length>11){
                    actEnd = 10
                    if(!mainView.bottomActionUp){
                        actEnd = sites.length
                        actStart = sites.length - 10
                    }
                }
                if(historyEnabled) {
                    var wayBackDate = util.getWayBackDate(currentWebview.url)
                    if( wayBackUrl.length==0 && wayBackDate.length>1){
                        wayBackUrl=util.getWayBackPreFix(currentWebview.url)
                        console.log("wayBackUrl: "+wayBackUrl)
                    } else if (wayBackDate.length==0){
                        historyEnabled=false
                        wayBackUrl=""
                    }
                }
                for (var i = actStart; i < actEnd; i++){
                    var title = sites[i].title
                    var linkUrl=sites[i].home
                    if(JS.bookmarkId(sites[i].position)){
                        title='>>> '+title
                    } 
                    if(historyEnabled && wayBackUrl.length>0 && util.getWayBackDate(linkUrl).length==0){
                        linkUrl = wayBackUrl + JS.getNonMobileUrl(linkUrl)
                    }                   
                    var text = ['import Ubuntu.Components 1.3; Action {onTriggered: {currentWebview.url="'
                               ,linkUrl,'"} text: "'
                               , title, '"; }'].join('')
                    actions.push(Qt.createQmlObject(text, bottomAct, "action"+sites[i].id+"_"+msec));
                }
                if(sites.length>11){
                    var text = 'import Ubuntu.Components 1.3; Action {'
                         +'onTriggered: {mainView.bottomActionUp=!mainView.bottomActionUp}'
                         +' text: mainView.bottomActionUp? ">>>": "<<<"; }'
                    actions.push(Qt.createQmlObject(text, bottomAct, "action_updown_"+msec));
                }
                return actions;
            }
        }
        Button {
            id: invisible
            height: units.gu(4)
            width: units.gu(0)
            anchors {
	            bottom: parent.bottom
                horizontalCenter: parent.horizontalCenter
            }
        }
        Button {
            id: backButton
            iconName: mainView.canShowIt(currentWebview.canGoBack) ? "go-previous" : ""
            width: mainView.canShowIt(currentWebview.canGoBack) ? units.gu(navButtonSize) : 0

            anchors {
	            bottom: parent.bottom
                left: parent.left
            }
	        onClicked: {
                currentWebview.goBack()
	        }
        }

        Button {
            id: nextButton
            iconName: mainView.canShowIt(currentWebview.canGoForward) ? "go-next" : ""
            width: mainView.canShowIt(currentWebview.canGoForward) ? units.gu(navButtonSize) : 0

            anchors {
	            bottom: parent.bottom
                right: parent.right
            }
	        onClicked: {
                 currentWebview.goForward()
	        }
        }
        states: [
            State {
                name: "menuSwitch"
                when: bottomActionUp
            }
        ]
        onStateChanged:{
            if(!appStartup){
                myTimer.delay(200, switchMenu)  
            }
        }
    }

    /* -------------------------- dialogue ---------------------------- */
    Item {
        width: units.gu(80)
        height: units.gu(80)
        Component {
             id: dialog
             Dialog {
                 id: dialogue
                 title: i18n.tr("Set the page as")
                 Button {
                     text: i18n.tr("app homepage")
                     color: UbuntuColors.green
                     onClicked: {
                         optionsKeeper.setAppHome(currentWebview.url)
                         PopupUtils.close(dialogue)
                     }
                 }
                 Button {
                     text: i18n.tr("website homepage")
                     color: UbuntuColors.green
                     onClicked: {
                          bookmarks.setHomePage(currentWebview.url)
                          PopupUtils.close(dialogue)
                     }
                 }
                 Button {
                     text: i18n.tr("bookmark")
                     color: UbuntuColors.green
                     onClicked: {
                          bookmarks.setBookmark(currentWebview.url)
                          PopupUtils.close(dialogue)
                     }
                 }
                 Button {
                     text: i18n.tr("cancel")
                     onClicked: {
                          PopupUtils.close(dialogue)
                     }
                 }
            }
        }
    }

    // ******************************** Add Website ***********************///////////////
    AddWebSitePage {
        id: addWebSitePage
        visible: false
    }
    // ******************************** Download Website ***********************///////////////

    DownloadWebSitePage {
        id: downloadWebSitePage
        visible: false
    }
    // ******************************** Website Management Page ***********************///////////////
    WebSiteManagement {
        id: webSiteManagement
        visible: false
    }
    // ******************************** Edit Website Page ***********************///////////////
    EditWebSitePage {
        id: editWebSite
        visible: false
    }
 
    // ******************************** Info Page ***********************///////////////
    InfoPage {
        id: infoPage
        visible: false
    }
    // ******************************** Settings Page ***********************///////////////
    SettingsPage {
        id: settingsPage
        visible: false
    }
    /* -------------------------- Utils ---------------------------- */

    OptionsKeeper {
        id: optionsKeeper
    }

    MyVersion {
        id: myVersion
    }

    Util {
        id: util
        webUA: myUA
        callerId: mainView
    }

    ParseRest {
        id: parseRest
    }

    Log2Parse {
        id: log
        app: mainView.applicationName
        revno: myVersion.revno
        parseConf: util.parseConf
    }

    Bookmarks {
        id: bookmarks
    }
    MyTimer {
       id: myTimer
    }   

    /* -------------------------- Components ---------------------------- */

    /* -------------------------- Connections ---------------------------- */
    Connections {
        target: Qt.inputMethod
        onVisibleChanged: nav.visible = !nav.visible
    }

/* does not work here... onFullscreenChanged not defined  

    Connections {
        target: webview
        onFullscreenChanged: nav.visible = !currentWebview.fullscreen
    }
*/
    Connections {
        target: UriHandler
        onOpened: {
            console.warn("uris is: " + uris[0]);
	        // only consider the first one (if multiple)
	        if (uris.length === 0 ) {
	            return;
	        }
	        currentWebview.url =  uris[0];
        }
    }

    Connections {
        target: settingsPage

        onSettingsChanged:{
            console.log("onSettingsChanged")
            mainView.getAppConfs()
        }
    }
}
