import QtQuick 2.4
// import Ubuntu.Components.Popups 1.3

import "util.js" as JS

Item {
    id:parseRest

    property string webSiteTitle: ""
    property string webSiteUrl: ""
    property string webSitePattern: ""
    property string domain
    property string categoryId
    property int subCategory
    property var oldObj
    property var objectName: "parseRest"

    signal parseReady()

     function init(parseData, title, siteUrl, pattern, category){
        webSiteTitle = title
        if(parseData){
            webSiteUrl = parseData.url
            domain = parseData.category
        } else {
            webSiteUrl = siteUrl
            domain = util.getDomain(siteUrl)
        }
        webSitePattern = pattern
        oldObj = parseData
        subCategory = category
    }

    function insertCategory(verifyResp){
        if(!JS.isInt(domain)){
            var uri = '/Category'
            var body = '{"name":"'+domain+'", "subCategory":['+subCategory+']}';
            var http = util.prepareRequest("POST", uri)
            http.onreadystatechange = function() { // Call a function when the state changes.
               if (http.readyState == 4) {
                   console.log("POST: " + http.responseText)
                   if (http.status == 201) {
                       console.log("status: " + http.status)
                       var resp = JSON.parse(http.responseText);
                       categoryId = resp.objectId
                       insertWebSite(verifyResp)
                   } else {
                       console.log("error: " + http.status)
                       util.popupError("http status="+http.status, http.responseText)
                   }
               }
            }
            http.send(body);
        } else {
            util.popupError("Bad url\n"+ webSiteUrl, "IP addresses are not allowed in URL")
        }
    }

    function insertSubCategory(){
        var uri = '/SubCategory'
        var body = '{"name":"'+JS.getCategoryName(subCategory)+'", "subCategoryId":'+subCategory+'}';
        var http = util.prepareRequest("POST", uri)
        http.onreadystatechange = function() { // Call a function when the state changes.
           if (http.readyState == 4) {
               console.log("POST: " + http.responseText)
               if (http.status == 201) {
                   manageWebSite()
               } else {
                   console.log("error: " + http.status)
                   util.popupError("http status="+http.status, http.responseText)
               }
           }
        }
        http.send(body);
    }

// webSite REST request body 
//    NOTE: url & domain not updatable!
//
    function doWebSiteBody(insert){
        var body = '{"name":"'+webSiteTitle+'","pattern":"'+webSitePattern
                  +'","subCategory":'+subCategory
        if(insert)body = body +',"url":"'+webSiteUrl+'", "category":"'+domain+'","safe":'+insert.safe+',"rank":"'+insert.rank+'"'
        if(JS.getAppTag() && JS.isDynamicAppTag())body = body +',"tags":{"__op":"AddUnique","objects":["'
                                                 +JS.getAppTag()+'"]}'
        body = body + '}'
        return body;     
    }

    function insertWebSite(verifyResp){
/* mrn 28/2/2016*/
/*        if(verifyResp && verifyResp.safe>util.parseConf.urlcheckMaxRcrds){
            util.popupError("Bad url\n"+ webSiteUrl, "Did not pass urlcheck.info safety test")
            log.message(log.warn, webSiteUrl+', safe: '+verifyResp.safe)
            util.urlCheckView(webSiteUrl)
        } else {
*/
            util.doRequest("POST", '/WebSite', doWebSiteBody(verifyResp))
            addSubCategory()
//        }
    }

    function addSubCategory(){
        var uri = '/Category/'+categoryId
        var body = '{"subCategory":{"__op":"AddUnique","objects":['+subCategory+']}'
        if(JS.getAppTag() && JS.isDynamicAppTag()){
            body = body + ',"tags":{"__op":"AddUnique","objects":["'+JS.getAppTag()+'"]}';
        }
        body = body + '}'
        var http = util.prepareRequest("PUT", uri)
        http.onreadystatechange = function() { // Call a function when the state changes.
           if (http.readyState == 4) {
               console.log("parseReady()")
               parseReady()
           }
        }
        http.send(body);
    }

    function updateCategory (){
        var uri = '/Category/?where='+encodeURIComponent('{"name":"'+domain+'"}');
        var http = util.prepareRequest("GET", uri)
        http.onreadystatechange = function() { // Call a function when the state changes.
            if (http.readyState == 4) {
                if (http.status == 200) {
                    var resp = JSON.parse(http.responseText);
                    if(resp.results.length>0){
                        categoryId = resp.results[0].objectId
                        util.verifyUrl(insertWebSite, webSiteUrl)
                    } else {
                        util.verifyUrl(insertCategory, webSiteUrl)
                    }
                } else {
                   console.log("error: " + http.status)
                   util.popupError("http status="+http.status, http.responseText)
                }
            }
        }
        http.send();
   }
   function removeSubCategory (){
        var uri = '/WebSite/?where='+encodeURIComponent('{"category":"'+domain+'", "subCategory":{"$in":['+oldObj.subCategory+']}}');
        var http = util.prepareRequest("GET", uri)
        http.onreadystatechange = function() { // Call a function when the state changes.
            if (http.readyState == 4) {
                if (http.status == 200) {
                    var resp = JSON.parse(http.responseText);
                    if(resp.results.length==0){
                        util.doRequest("PUT", '/Category/'+categoryId,
                                 '{"subCategory":{"__op":"Remove","objects":['+oldObj.subCategory+']}}')
                    }
                } else {
                   console.log("error: " + http.status)
                   util.popupError("http status="+http.status, http.responseText)
                }
            }
        }
        http.send();
   }
   function updateSubCategories (){
        var uri = '/Category/?where='+encodeURIComponent('{"name":"'+domain+'"}');
        var http = util.prepareRequest("GET", uri)
        http.onreadystatechange = function() { // Call a function when the state changes.
            if (http.readyState == 4) {
                if (http.status == 200) {
                    var resp = JSON.parse(http.responseText);
                    if(resp.results.length>0){
                        categoryId = resp.results[0].objectId
                        addSubCategory()
                        if(subCategory!=oldObj.subCategory){
                             removeSubCategory()
                        }
                    } else {
                        var body = '{"name":"'+domain+'", "subCategory":['+subCategory+']}';
                        util.doRequest("POST",'/Category',body)
                    }
                } else {
                   console.log("error: " + http.status)
                   util.popupError("http status="+http.status, http.responseText)
                }
            }
        }
        http.send();
   }

   function uploadWebSite (){
        var uri = '/SubCategory/?where='+encodeURIComponent('{"subCategoryId":'+subCategory+'}');
        var http = util.prepareRequest("GET", uri)
        http.onreadystatechange = function() { // Call a function when the state changes.
            if (http.readyState == 4) {
                if (http.status == 200) {
                    var resp = JSON.parse(http.responseText);
                    if(resp.results.length>0){
                        manageWebSite()
                    } else {
                        insertSubCategory()
                    }
                } else {
                   console.log("error: " + http.status)
                   util.popupError("http status="+http.status, http.responseText)
                }
            }
        }
        http.send();
   }

   function manageWebSite(){
       if(oldObj){
            var uri = '/WebSite/'+oldObj.objectId;
            var http = util.prepareRequest("PUT", uri)
            http.onreadystatechange = function() { // Call a function when the state changes.
                if (http.readyState == 4) {
                    if (http.status == 200) {
                        console.log(http.responseText);
                        updateSubCategories ()
                    } else {
                       console.log("error: " + http.status)
                       util.popupError("http status="+http.status, http.responseText)
                    }
                }
            }
            http.send(doWebSiteBody());

        } else {
           updateCategory()
       }
   }
//
// get configuration....
//

    function include(arr,obj) {
        return (arr.indexOf(obj) != -1);
    }

   function getMoreWebSites(callback){
       var conf = arguments[1]
       console.log("conf="+JSON.stringify(conf))
       var webSites
       if (conf.webSites && conf.webSites.length>0){
           webSites = conf.webSites
       } else {
           webSites = []
       }
       var limit = conf.maxSites - webSites.length
       var filter = '{"category":{"$in":'+JSON.stringify(conf.categories)+'}'
       if(include(conf.subCategories,0)){
           filter = filter + ',"subCategory":{"$in":'+JSON.stringify(JS.getCategoryIds())+'}'
       } else { 
           filter = filter + ',"subCategory":{"$in":'+JSON.stringify(conf.subCategories)+'}'
       }
       if(conf.tags && conf.tags.length>0){
           filter = filter + ',"tags":{"$all":'+JSON.stringify(conf.tags)+'}'
       }
       filter = filter + '}'
       console.log("filter="+filter)
       var uri = '/WebSite/?where='+encodeURIComponent(filter)+'&limit='+limit;
       console.log(uri)
       var http = util.prepareRequest("GET", uri)
       http.onreadystatechange = function() { // Call a function when the state changes.
           if (http.readyState == 4) {
               if (http.status == 200) {
                   var resp = JSON.parse(http.responseText);
                   if(resp.results.length>0){
                       console.log("more webSites found! "+resp.results.length)
                       for(var i=0; i<resp.results.length; i++){
                           webSites.push(resp.results[i].objectId)
                       }
                   } else {
                       console.log("ERROR: no webSites found!")
                   }
                   getWebSites(callback, webSites, conf.order)
               } else {
                  console.log("error: " + http.status)
                  util.popupError("http status="+http.status, http.responseText)
               }
           }
       }
       http.send();
   }

   function getWebSites(callback){
       var ids = arguments[1]
       var order = arguments[2]
       var uri = '/WebSite/?where='+encodeURIComponent('{"objectId":{"$in":'+JSON.stringify(ids)+'}}')
       if(order){
           uri = uri + '&order='+order;
       }
       console.log(uri)
       var http = util.prepareRequest("GET", uri)
       http.onreadystatechange = function() { // Call a function when the state changes.
           if (http.readyState == 4) {
               if (http.status == 200) {
                   var resp = JSON.parse(http.responseText);
                   if(resp.results.length>0){
                       console.log("webSites found! "+resp.results.length)
                       callback(resp.results)
                   } else {
                       log.message(log.error, JSON.stringify(ids)+', order: '+order)
                       console.log("ERROR: no webSites found!")
                   }
               } else {
                   console.log("error: " + http.status)
                   util.popupError("http status="+http.status, http.responseText)
               }
           }
       }
       http.send();
   }

   function getConfiguration(callback){
       var filter = arguments[1]
       var i = arguments[2]
       var uri = '/Configuration/?where='+encodeURIComponent(filter[i]);
       console.log(uri)
       var http = util.prepareRequest("GET", uri)
       http.onreadystatechange = function() { // Call a function when the state changes.
           if (http.readyState == 4) {
               if (http.status == 200) {
                   var resp = JSON.parse(http.responseText);
                   if(resp.results.length>0){
                       console.log("configuration found!")
                       if(!resp.results[0].categories || !resp.results[0].subCategories){
                           getWebSites(callback, resp.results[0].webSites, resp.results[0].order)
                       } else {
                           getMoreWebSites(callback, resp.results[0])
                       }
                   } else {
                       i++
                       if(i<filter.length){
                           getConfiguration(callback, filter, i)
                       } else {
                           console.log("ERROR: no configuration found!")
                       }
                   }
               } else {
                  console.log("error: " + http.status)
                  util.popupError("http status="+http.status, http.responseText)
               }
           }
       }
       http.send();
   }
 
   function getDefWebSites(callback){
       var locale = util.getLocale()
       if(!locale)locale="null"
       var loc = JS.splitLocale(locale)
       if(loc.length==1)loc.push("null")
       log.message(log.info, JSON.stringify(loc))
       var filter=[]
       var app = JS.getWebappName()
       filter.push('{"language":"'+loc[0]+'","country":"'+loc[1]+'","app":"'+app+'"}') 
       filter.push('{"language":"null","country":"'+loc[1]+'","app":"'+app+'"}') 
       filter.push('{"language":"'+loc[0]+'","country":"null","app":"'+app+'"}') 
       filter.push('{"language":"null","country":"null","app":"'+app+'"}') 
       getConfiguration(callback, filter, 0)
   }
//
// search webSite configuration....
//
   function searchConfiguration(callback){
       var webSiteUrl = arguments[1]
        var urls = JS.getEquivalentUrls(webSiteUrl)
        var filter
        if(urls.length>1){
            filter='{"$or":[{"url":"'+urls[0]+'"}'
            for (var i = 1; i < urls.length; i++){
                filter=filter+',{"url":"'+urls[i]+'"}'
            }
            filter=filter+']}'

        } else {
            filter = '{"url":"'+urls[0]+'"}' 
        }
        console.log("filter: " +filter)
        var uri = '/WebSite/?where='+encodeURIComponent(filter);
        var http = util.prepareRequest("GET", uri)
        http.onreadystatechange = function() { // Call a function when the state changes.
            if (http.readyState == 4) {
                if (http.status == 200) {
                    var resp = JSON.parse(http.responseText);
                    var parseData = null
                    if(resp.results.length>0){
                        parseData = resp.results[0]
                        if(parseData.duplicate){ 
                            newConfiguration(callback, parseData.duplicate)  // url changed!
                        } else {
                            callback(parseData)
                        }
                    } else {
                        callback(parseData)
                    }
                } else {
                   console.log("error: " + http.status)
                   util.popupError("http status="+http.status, http.responseText)
                }
            }
        }
        http.send();
    }

    function newConfiguration(callback){
        var id = arguments[1]
        var filter = '{"objectId":"'+id+'"}' 
        console.log("filter: " +filter)
        var uri = '/WebSite/?where='+encodeURIComponent(filter);
        var http = util.prepareRequest("GET", uri)
        http.onreadystatechange = function() { // Call a function when the state changes.
            if (http.readyState == 4) {
                if (http.status == 200) {
                    var resp = JSON.parse(http.responseText);
                    var parseData = null
                    if(resp.results.length>0){
                        parseData = resp.results[0]
                        parseData.duplicate = "true"
                    }
                    callback(parseData)
                } else {
                   console.log("error: " + http.status)
                   util.popupError("http status="+http.status, http.responseText)
                }
            }
        }
        http.send();
    }

}

