import QtQuick 2.4
import Ubuntu.Components 1.3
import "databasemodule_v2.js" as DB
import "util.js" as JS

Page {
    id: reorderWebSitesPage
    objectName: "reorderWebSitesPage"

    title: i18n.tr("Setup")
    flickable: content
    property var newOrder: []

    head.backAction: Action {
        iconName: "close"
        onTriggered: {
            var oldpos
            var orig = DB.loadWebSites().rows

            for(var i=0; i<newOrder.length; i++){
                var o = newOrder[i]
                var found=-1
                for(var j=0; j<orig.length; j++){
                    if(o.id==orig[j].id){
                        oldpos = o.position
                        if(oldpos==i)break
                        found=0
                    }
                    if (found>-1){
                        if(oldpos==JS.positionId(orig[j].position)){
                            var newpos = i + found/100
                            DB.updateWebSitePosition(orig[j].id, newpos)
                            found++
                        } else {
                            break
                        }
                    }
                }
            }
            webSiteManagement.isReordined=true
            pageStack.pop()
        }
    }
    Component.onCompleted: {
        refresh()
    }
    function refresh(){
        var ws = DB.loadWebSites().rows
        webSiteModel.clear()
        for (var i=0; i < ws.length; i++){
            if(!JS.bookmarkId(ws[i].position)){
                webSiteModel.append(
                {
                    "id": ws[i].id
                    ,"title": ws[i].title
                    ,"position": ws[i].position
                })
            }
        }
    }
    /*
      main content
    */
    Flickable {
        id: content

        anchors {
            left: parent.left
            right: parent.right
            top: parent.top
            bottom: parent.bottom
        }
        contentHeight: contentItem.childrenRect.height
 
        Column{
            anchors{ left: parent.left; right: parent.right }

            /*
              WebSite listview
            */

            ListView {
                id: webSiteList
                anchors{ left: parent.left; right: parent.right }
                height: contentItem.childrenRect.height
                interactive: false
                currentIndex: -1
                ViewItems.dragMode : true

                model: ListModel {
                    id: webSiteModel
                }
                delegate: ListItem {
                    Label {
                        text: model.title
                    }
                }
                ViewItems.onDragUpdated: {
                    if (event.status == ListItemDrag.Moving) {
                        // inform dragging that move is not performed
                        event.accept = false;
                    } else if (event.status == ListItemDrag.Dropped) {
                        console.log(event.from+'->'+event.to+' title: '+model.get(event.from).title)
                        model.move(event.from, event.to, 1);
                        for (var i=0; i < model.count; i++){
                            var o = {"id":model.get(i).id
                                    ,"position": model.get(i).position}
                            newOrder[i] = o
                        }
                    }
                }
                moveDisplaced: Transition {
                    UbuntuNumberAnimation {
                        property: "y"
                    }
                }
            }
        }
    }
}

