/*
 * Copyright (C) 2013, 2014
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import QtQuick 2.4
import Ubuntu.Components 1.3
import "util.js" as JS
import "../webArchive.js" as WebArchive

Page {
    id: settingsPage
    objectName: "settingsPage"
    title: i18n.tr("App Settings")
    flickable: content

    property var allowPopup
    property var navButtons
    property var gTLDfilter
    property var langFilter
    property string domainFilter
    property var dwnPatterns
    property var secondaryPatterns: []
    property var bmtrUseHttpReq
    property var historyButton
    property bool oskVisible: false
 
    signal settingsChanged()
    signal filterChanged()

    Component.onCompleted: {
        allowPopup = optionsKeeper.allowPopup()
        navButtons = optionsKeeper.navButtons()
        gTLDfilter = optionsKeeper.gTLDfilter()
        langFilter = optionsKeeper.langFilter()
        myDomains.text = optionsKeeper.myFilter()
        myPatterns.text = optionsKeeper.myPatterns()
        dwnPatterns = optionsKeeper.useDownloadedPatterns()
        bmtrUseHttpReq = optionsKeeper.bmtrUseHttpReq()
        historyButton = optionsKeeper.historyButton()
        navButtonStyle.selectedIndex = optionsKeeper.navButtonSize()
        if(util.gTLD[0])calculateFilter()
        calculatePatterns()
        console.log('settingsPage completed')
    }

    function calculateFilter(){
        var currFilterA = []
        if(gTLDfilter)currFilterA = util.getgTLD()
        if(langFilter)currFilterA = JS.arrayUnique(currFilterA.concat(JS.defaultDomains(Qt.locale().name)));
        if(myDomains.text){
            var myd = myDomains.text.split(',')
            currFilterA = JS.arrayUnique(currFilterA.concat(myd));
        }
        domainFilter= JSON.stringify(currFilterA)
        console.log("calculateFilter: "+domainFilter)
        filterChanged()
    }
    function calculatePatterns(){
        console.log("calculatePatterns...")
        secondaryPatterns = []
        if(dwnPatterns) {
            secondaryPatterns = util.getSecondaryPatterns()
        }
        if(historyButton){
            console.log("add historybutton "+historyButton)
            var webArchive = []
            webArchive.push(WebArchive.pattern)
            secondaryPatterns = JS.arrayUnique(secondaryPatterns.concat(webArchive));
        }
        if(myPatterns.text){
            var myp = myPatterns.text.split(',')
            secondaryPatterns = JS.arrayUnique(secondaryPatterns.concat(myp));
        }
        console.log("secondaryPatterns: "+secondaryPatterns)
    }
    function doUpdate(){
        console.log('settings update...')
        optionsKeeper.setMyFilter(myDomains.text)
        optionsKeeper.setMyPatterns(myPatterns.text)
        calculateFilter()
        calculatePatterns()
    }
    function navButtonConfigured(index){
        console.log('navButtonStyle: '+index)
        optionsKeeper.setNavButtonSize(index)
        if(index==3){
            optionsKeeper.setNavButtons(false)
        } else {
            optionsKeeper.setNavButtons(true)
        }
        settingsChanged()
    }
    Flickable {
        id: content
        anchors {
            left: parent.left
            right: parent.right
            top: parent.top
            bottom: fakePanel.top
        }
        contentHeight: contentItem.childrenRect.height
        Column
        {
            anchors{ left: parent.left; right: parent.right }
            spacing: units.gu(1)

            Row
            {
                anchors{ left: parent.left; right: parent.right; leftMargin: units.gu(2); rightMargin: units.gu(2) }
                spacing: units.gu(1)

                Text
                {
                    id: mainViewTitle
                    objectName: "mainViewTitle"
                    text: i18n.tr("Main View:")
                    anchors.verticalCenter: parent.verticalCenter
                    wrapMode: Text.WordWrap
                    width: parent.width - parent.spacing
                    font.bold: true

                }
            }
/*
            Row
            {
                anchors{ left: parent.left; right: parent.right; leftMargin: units.gu(3); rightMargin: units.gu(2) }
                spacing: units.gu(1)
                Text
                {
                    id: setNavButtons
                    objectName: "setNavButtons"
                    text: i18n.tr("Use navigation buttons")
                    width: parent.width - parent.spacing - units.gu(5)
                    anchors.verticalCenter: parent.verticalCenter
                }
                Switch {
                    checked: navButtons
                    onClicked: {
                        console.log("clicked: "+ checked)
                        optionsKeeper.setNavButtons(checked)
                        settingsChanged()
                    }
                }
            }
*/

            Row
            {
                anchors{ left: parent.left; right: parent.right; leftMargin: units.gu(3); rightMargin: units.gu(2) }
                spacing: units.gu(1)
                OptionSelector {
                    id: navButtonStyle
                    objectName: "navButtonConfiguration"
                    text: i18n.tr("Navigation buttons:")
                    expanded: true
                    multiSelection: false
                    containerHeight: itemHeight * 4

                    model: [i18n.tr("small"),
                            i18n.tr("medium"),
                            i18n.tr("large"),
                            i18n.tr("radial")]
                    onDelegateClicked: navButtonConfigured(index)
                 }      
            }
            Row
            {
                anchors{ left: parent.left; right: parent.right; leftMargin: units.gu(3); rightMargin: units.gu(2) }
                spacing: units.gu(1)
                Text
                {
                    id: setHistoryButton
                    objectName: "setHistoryButton"
                    text: i18n.tr("Enable history navigation")
                    width: parent.width - parent.spacing - units.gu(5)
                    anchors.verticalCenter: parent.verticalCenter
                }
                Switch {
                    checked: historyButton
                    onClicked: {
                        optionsKeeper.setHistoryButton(checked)
                        historyButton=checked
                        calculatePatterns()
                        settingsChanged()
                    }
                }
            }

            Row
            {
                anchors{ left: parent.left; right: parent.right; leftMargin: units.gu(3); rightMargin: units.gu(2) }
                spacing: units.gu(1)
                Text
                {
                    id: allowPopups
                    objectName: "allowPopups"
                    text: i18n.tr("Allow overlay windows")
                    width: parent.width - parent.spacing - units.gu(5)
                    anchors.verticalCenter: parent.verticalCenter
                }
                Switch {
                    checked: allowPopup
                    onClicked: {
                        console.log("clicked: "+ checked)
                        optionsKeeper.setAllowPopup(checked)
                        settingsChanged()
                    }
                }
            }
            Row
            {
                anchors{ left: parent.left; right: parent.right; leftMargin: units.gu(3); rightMargin: units.gu(2) }
                spacing: units.gu(1)
                Text
                {
                    id: useHttpReqs
                    objectName: "useHttpReqs"
                    text: i18n.tr("Use the net to solve bookmark titles")
                    width: parent.width - parent.spacing - units.gu(5)
                    anchors.verticalCenter: parent.verticalCenter
                }
                Switch {
                    checked: bmtrUseHttpReq
                    onClicked: {
                        console.log("clicked: "+ checked)
                        optionsKeeper.setBmtrUseHttpReq(checked)
                    }
                }
            }
            Row
            {
                anchors{ left: parent.left; right: parent.right; leftMargin: units.gu(2); rightMargin: units.gu(2) }
                spacing: units.gu(1)

                Text
                {
                    id: filterTitle
                    objectName: "filterTitle"
                    text: i18n.tr("Top Level Domain filter:")
                    anchors.verticalCenter: parent.verticalCenter
                    wrapMode: Text.WordWrap
                    width: parent.width - parent.spacing
                    font.bold: true

                }
            }
            Row
            {
                anchors{ left: parent.left; right: parent.right; leftMargin: units.gu(3); rightMargin: units.gu(2) }
                spacing: units.gu(1)
                Text
                {
                    id: gTLD
                    objectName: "gTLD"
                    text: i18n.tr("Add generic Top Level Domains")
                    width: parent.width - parent.spacing - units.gu(5)
                    anchors.verticalCenter: parent.verticalCenter
                }
                Switch {
                    checked: gTLDfilter
                    onClicked: {
                        console.log("gTLD: "+ checked)
                        gTLDfilter=checked
                        optionsKeeper.setgTLDfilter(checked)
                        calculateFilter()
                    }
                }
            }
            Row
            {
                anchors{ left: parent.left; right: parent.right; leftMargin: units.gu(3); rightMargin: units.gu(2) }
                spacing: units.gu(1)
                Text
                {
                    id: syslang
                    objectName: "syslang"
                    text: i18n.tr("Use system language")
                    width: parent.width - parent.spacing - units.gu(5)
                    anchors.verticalCenter: parent.verticalCenter
                }
                Switch {
                    checked: langFilter
                    onClicked: {
                        console.log("syslang: "+ checked)
                        langFilter=checked
                        optionsKeeper.setLangFilter(checked)
                        calculateFilter()
                    }
                }
            }
            Row
            {
                anchors{ left: parent.left; right: parent.right; leftMargin: units.gu(3); rightMargin: units.gu(1) }
                spacing: units.gu(1)

                TextField
                {
                    id: myDomains
                    objectName: "myDomains"
                    placeholderText: i18n.tr("Domains separated by \",\" i.e. fi,Finnish")
                    width: parent.width - parent.spacing
                    anchors.verticalCenter: parent.verticalCenter
                    readOnly: false
                    hasClearButton: false
                }
            }
            Row
            {
                anchors{ left: parent.left; right: parent.right; leftMargin: units.gu(3); rightMargin: units.gu(2) }
                spacing: units.gu(1)

                Text
                {
                    id: currentFilter
                    objectName: "currentFilter"
                    text: domainFilter 
                    anchors.verticalCenter: parent.verticalCenter
                    wrapMode: Text.WrapAnywhere
                    width: parent.width - parent.spacing

                }
            }
            Row
            {
                anchors{ left: parent.left; right: parent.right; leftMargin: units.gu(2); rightMargin: units.gu(2) }
                spacing: units.gu(1)

                Text
                {
                    id: patternTitle
                    objectName: "patternTitle"
                    text: i18n.tr("Secondary URL patterns:")
                    anchors.verticalCenter: parent.verticalCenter
                    wrapMode: Text.WordWrap
                    width: parent.width - parent.spacing
                    font.bold: true

                }
            }
            Row
            {
                anchors{ left: parent.left; right: parent.right; leftMargin: units.gu(3); rightMargin: units.gu(2) }
                spacing: units.gu(1)
                Text
                {
                    id: useDwndPatterns
                    objectName: "useDwndPatterns"
                    text: i18n.tr("Use downloaded patterns")
                    width: parent.width - parent.spacing - units.gu(5)
                    anchors.verticalCenter: parent.verticalCenter
                }
                Switch {
                    checked: dwnPatterns
                    onClicked: {
                        console.log("clicked: "+ checked)
                        dwnPatterns = checked
                        optionsKeeper.setUseDownloadedPatterns(checked)
                        calculatePatterns()
                    }
                }
            }
            Row
            {
                anchors{ left: parent.left; right: parent.right; leftMargin: units.gu(3); rightMargin: units.gu(1) }
                spacing: units.gu(1)

                TextField
                {
                    id: myPatterns
                    objectName: "myPatterns"
                    placeholderText: i18n.tr("URL patterns separated by \",\"")
                    width: parent.width - parent.spacing
                    anchors.verticalCenter: parent.verticalCenter
                    readOnly: false
                    hasClearButton: false
                }
            }
            Row
            {
                anchors{ left: parent.left; right: parent.right; leftMargin: units.gu(3); rightMargin: units.gu(2) }
                spacing: units.gu(1)

                Text
                {
                    id: currentPattern
                    objectName: "currentPattern"
                    text: JSON.stringify(secondaryPatterns) 
                    anchors.verticalCenter: parent.verticalCenter
                    wrapMode: Text.WrapAnywhere
                    width: parent.width - parent.spacing
                }
            }
        }
    }
    Rectangle {
        id: fakePanel

        color: "#fafafa"
        anchors {
            left: parent.left
            right: parent.right
            bottom: parent.bottom
        }
        height: units.gu(6)
        Item {
            anchors.fill: parent

            Button {
                id: actionsButton
                text: i18n.tr("Update")
                anchors {
                    verticalCenter: parent.verticalCenter
                    right: parent.right
                    rightMargin: units.gu(1)
                }
                onClicked: {
                    if(!oskVisible){
                        doUpdate()
                    }
                }
            }
        }
    } // Rectangle fakePanel
    onStateChanged:{
        if (actionsButton.pressed && oskVisible){
            util.click()
            doUpdate()
        }
    }
    states: [
        State {
            name: "pressed"
            when: actionsButton.pressed
        }
    ]

    Connections {
        target: Qt.inputMethod
        onVisibleChanged: {
            oskVisible=!oskVisible
        }
    }
    Connections {
        target: util

        onGTLDready:{
            console.log("onGTLDready: "+util.getgTLD())
            calculateFilter()
        }
    }
    Connections {
        target: mainView

        onSettingsUpdated:{
            console.log("onFilterUpdated")
            gTLDfilter = optionsKeeper.gTLDfilter()
            langFilter = optionsKeeper.langFilter()
            allowPopup = optionsKeeper.allowPopup()
            myPatterns.text = optionsKeeper.myPatterns()
            dwnPatterns = optionsKeeper.useDownloadedPatterns()
            bmtrUseHttpReq = optionsKeeper.bmtrUseHttpReq()
            historyButton = optionsKeeper.historyButton()
            doUpdate()
        }
    }

    Connections {
        target: util

        onPatternsUpdated:{
            console.log("onPatternsUpdated")
            calculatePatterns()
        }
    }

}
