import QtQuick 2.4
import QtFeedback 5.0
import Ubuntu.Components.Popups 1.3
import "util.js" as JS
import "databasemodule_v2.js" as DB
import "../config.js" as Conf
import "../webArchive.js" as WebArchive

Item {
    id: util
    property var objectName: "util"

    property var gTLD: []
    property var secondaryPatterns: []
    property string appKey: ""
    property string clientKey: ""
    property var parseConf
    property var callerId
    property var webUA

    signal websiteUpdated()
    signal patternsUpdated()
    signal gTLDready()

    HapticsEffect {
        id: clickEffect
        attackIntensity: 0.0
        attackTime: 50
        intensity: 1.0
        duration: 10
        fadeTime: 50
        fadeIntensity: 0.0
    }

    function click(){
       clickEffect.start() 
    }

    function init(){
//        testSHA2()
        initParseConf()
        updateSecondaryPatterns()
    }
    function getgTLD(){
        return gTLD 
    }
    function getSecondaryPatterns(){
        return secondaryPatterns 
    }

    function getDomain(href){
        var domain = JS.getLocation(href).host.split('.').pop().toLowerCase().split(':')[0]
        console.log('gTLD: '+gTLD)
        if(JS.contains(gTLD, domain) || domain.length>3){
            var lang  = JS.getTranslation(href)
            if(lang){
                console.log("language/territory: "+lang)
                domain=lang
            }
        }
        return domain;
    }
    function getAppKey(){
        if(appKey === "") {
            doKeys()        
        }
        return appKey;
    }
    function getClientKey(){
        if(clientKey === "") {
            doKeys()        
        }
        return clientKey;
    }

    function getLocale(){
       var locale = Qt.locale().name
       if(Conf.debugLocale)locale=Conf.debugLocale
       return locale
    }

    function doKeys(){
        var k = decodeBase64(Conf.parseCom).replace(/\r?\n/g, "").split(Conf.webappName)
        appKey = k[0]
        clientKey = k[1]
    }
    function decodeBase64 (s) {
        var e={},i,b=0,c,x,l=0,a,r='',w=String.fromCharCode,L=s.length;
        var A="ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
        for(i=0;i<64;i++){e[A.charAt(i)]=i;}
        for(x=0;x<L;x++){
            c=e[s.charAt(x)];b=(b<<6)+c;l+=6;
            while(l>=8){((a=(b>>>(l-=8))&0xff)||(x<(L-2)))&&(r+=w(a));}
        }
        return r;
    }

    function prepareRequest (type, uri){
        var http = new XMLHttpRequest()
        http.open(type, 'https://api.parse.com/1/classes'+uri);
        http.setRequestHeader("X-Parse-Application-Id", getAppKey());
        http.setRequestHeader("X-Parse-REST-API-Key", getClientKey());
        if(type != "GET"){
            http.setRequestHeader("Content-Type", "application/json");
            http.setRequestHeader("charset", "utf-8");
        }
        return http;
    }

    function initParseConf (){
        if(optionsKeeper.parseConf()){
            parseConf = JSON.parse(optionsKeeper.parseConf())
        }
        var http = new XMLHttpRequest()
        http.open('GET', 'https://api.parse.com/1/config');
        console.log("parseConf: getConf...")
        http.setRequestHeader("X-Parse-Application-Id", getAppKey());
        http.setRequestHeader("X-Parse-REST-API-Key", getClientKey());
        http.onreadystatechange = function() { // Call a function when the state changes.
           if (http.readyState == 4) {
               if (http.status == 200 || http.status == 201) {
                   parseConf = JSON.parse(http.responseText).params;
                   optionsKeeper.setParseConf(JSON.stringify(parseConf)) 
                   console.log("parseConf initialized")
                   var updated = new Date(parseConf.bmtrUpdated.iso);
                   bookmarks.init(updated.getTime())
                   updated = new Date(parseConf.gtldUpdated.iso);
                   if(!optionsKeeper.gtldDownloaded() || optionsKeeper.gtldDownloaded()<updated){
                       console.log("download a new generic top level domains")
                       doRequestCallback(setgTLD, 'GET',
                             '/Category/?where='+encodeURIComponent('{"type":1}'))
                   } else {
                       gTLD = JSON.parse(optionsKeeper.gTLD());
                       gTLDready()
                   }
               } else {
                   console.log("ERROR: getParseConf failed! Http status="+http.status)
               }
           }
        }
        http.send(); 
    }
    function setgTLD(resp){
        var currentdate = new Date();
        gTLD =[]
        for (var i = 0; i < resp.results.length; i++){
            gTLD.push(resp.results[i].name)
        }
        optionsKeeper.setGtldDownloaded(currentdate.getTime());
        gTLDready()
        optionsKeeper.setGTLD(JSON.stringify(gTLD)) 
    }

    function testSHA2 (){
        var http = new XMLHttpRequest()
        http.open('GET', 'https://sha2-test.parse.com');
        console.log("testSHA2...")
        http.onreadystatechange = function() { // Call a function when the state changes.
           if (http.readyState == 4) {
               if (http.status == 200 || http.status == 201) {
                   console.log("testSHA2 response: "+http.responseText)
               } else {
                   console.log("ERROR: testSHA2 failed! Http status="+http.status)
               }
           }
        }
        http.send(); 
    }

    function doRequestCallback(callback){
        var type = arguments[1]
        var uri = arguments[2]
        var body = arguments[3]
        var userData = arguments[4]
        console.log('doRequestCallback('+type+', '+uri+', '+body+')')
        var http = prepareRequest(type, uri)
        http.onreadystatechange = function() { // Call a function when the state changes.
           if (http.readyState == 4) {
               console.log("status: " + http.status)
               if (http.status == 200 || http.status == 201) {
                   var resp = JSON.parse(http.responseText)
                   if(resp){
                       callback(resp, userData)
                   } else {
                       console.log("bad response: "+http.responseText)
                   }
               } else {
                   popupError("http status="+http.status, http.responseText)
               }
           }
        }
        http.send(body);
    }

    function doRequest(type, uri, body){
        var http = prepareRequest(type, uri)
        http.onreadystatechange = function() { // Call a function when the state changes.
           if (http.readyState == 4) {
               if (http.status == 200 || http.status == 201) {
                   console.log("status: " + http.status)
               } else {
                   console.log("error: " + http.status)
                   popupError("http status="+http.status, http.responseText)
               }
           }
        }
        http.send(body);
    }

    function popupMessage(title, message){
        PopupUtils.open(Qt.resolvedUrl("QrErrorDialog.qml"), callerId, {
            title: title,
            message: message 
        })
    }

    function popupError(title, message){
        popupMessage(title, message)
    }

    function nslookup(callback){
        var url = arguments[1]
        var host = JS.getLocation(url).host
        var ws = {}
        ws.url = url
        console.log('nslookup '+host)
        var http = new XMLHttpRequest()
        http.open('GET', 'http://ip-api.com/json/'+host);
        http.onreadystatechange = function() { // Call a function when the state changes.
           if (http.readyState == 4) {
               console.log("status: " + http.status)
               if (http.status == 200 || http.status == 201) {
                   var resp = JSON.parse(http.responseText);
                   if(resp.status=="success"){
                       urlCheckCallback(callback, ws)
                   } else {
                       log.message(log.error, url+', message='+resp.message)
                       popupError("Bad url\n"+ webSiteUrl, resp.message)
                   }
               } else {
                   console.log("nslookup failed!")
                   urlCheckCallback(callback, ws)
               }
           }
        }
        http.send(); 
    }

    function pingUrl(callback){
        var url = arguments[1]
        console.log('ping('+url+')')
        var http = new XMLHttpRequest()
        http.open('GET', url);
        http.setRequestHeader("Host", JS.getLocation(url).host);
        http.setRequestHeader("User-Agent", webUA); 
        http.onreadystatechange = function() { // Call a function when the state changes.
           if (http.readyState == 4) {
               console.log("status: " + http.status)
               if (http.status == 200 ) {
                   callback(url)
               }
           }
        }
        http.send();
    }


    function verifyUrl(callback){
        var url = arguments[1]
        console.log('ping('+url+')')
        var http = new XMLHttpRequest()
        http.open('GET', url);
        http.setRequestHeader("Host", JS.getLocation(url).host);
        http.setRequestHeader("User-Agent", webUA); 
        http.onreadystatechange = function() { // Call a function when the state changes.
           if (http.readyState == 4) {
               console.log("status: " + http.status)
               console.log("status: " + http.statusText)
               if (http.status == 200 ) {
                   var ws = {}
                   ws.url = url
                   urlCheckCallback(callback, ws)
               } else if(http.status == 0 || http.status == 406 || http.status == 403){
// mrn 14/2/2016: added status 406
//     28/2/2016: added status 403
// W/A: to fix bug...
// https://bugs.launchpad.net/my-webapp-group/+bug/1478327
//
                   log.message(log.warn, url)
                   nslookup(callback, url)
               } else {
                   log.message(log.error, url+', status='+http.status)
                   popupError("Bad url\n"+ url, "http status="+http.status+"\n"+http.getAllResponseHeaders())
               }
           }
        }
        http.send();
    }

    function urlCheck(website){
//mrn 6/3/2016: check & update website allways from user action
//        if(JS.daysAgo(website.updatedAt)>0){
            urlCheckCallback(updateWebsite, website) 
//        }
    }

    function urlCheckCallback(callback){
        var website = arguments[1]
        website.safe=-1
        var url = website.url
        console.log('urlCheck '+url)
        var http = new XMLHttpRequest()
        http.open('GET', parseConf.urlcheckURL+url);
        http.onreadystatechange = function() { // Call a function when the state changes.
            if (http.readyState == 4) {
                console.log("status: " + http.status)
                if (http.status == 200 || http.status == 201) {
                     var resp = http.responseText;
                     var regex = new RegExp(parseConf.urlcheckRegex);
                     var match = resp.match(regex)
                     if(match){
                         console.log("match: "+match)
//                         website.message=match[parseConf.urlcheckMessage]
                         website.message="SUCCESS: urlcheck.info response parsed correctly!" 
                         website.safe=match[parseConf.urlcheckStatus]
                     } else {
                         log.message(log.error, website.url)
                         website.message="ERROR: urlcheck.info result not found!"
                         console.log(website.message)
                     }
                     if(parseConf.rank2Loc1>0){
//                         var alexaRankRegex1="(.*)this URL is <b>(.*)<\/b> and (.*)"
                         regex = new RegExp(parseConf.rankRegex1)
                         match = resp.match(regex)
                         var rank=""
                         if(match){
                             var n = match[parseConf.rank1Loc].replace(",","")*1
                             if(n<100000){
                                 rank = match[parseConf.rank1Loc]+""
                             }
                         }
                         if(rank.length>4 || rank.length===0){                      
//                             var alexaRankRegex2="(.*)for the country <strong>(.*) (.*)<\/strong> is this <b>(.*)<\/b>.(.*)"
                             regex = new RegExp(parseConf.rankRegex2)
                             match = resp.match(regex)
                             if(match){
                                 var n = match[parseConf.rank2Loc1].replace(",","")*1
                                 var m = 1000000
                                 if(rank.length!==0) m = rank.replace(",","")*1
                                 if(n<100000 && n<m){
                                     rank = match[parseConf.rank2Loc1]+match[parseConf.rank2Loc2]
                                 }                            
                             }
                         }
                         if(rank.length===0){
                             rank="N/A"
                             if(website.rank && website.rank !== "N/A" && website.rank.length>0){
                                 log.message(log.warn, website.url)
                                 console.log("WARN: website alexa rank not found!")
                             }
                         }
                         website.rank=rank                
                         console.log("rank: "+rank)
                    }
                } else {
                     website.message="ERROR: urlcheck.info bad response"
                     console.log(website.message)
                }
                callback(website)
            }
        }
        http.send(); 
    }
    function showRank(){
        if(parseConf.rank1Loc>0) return true
        return false
    }
    function urlCheckView(url){
        var locale = getLocale()
        var loc = JS.splitLocale(locale)
        var l = "/en/"
        if(loc[0]=='it'){
            l ="/it/"
        } else if(loc[0]=='es'){
            l ="/es/"
        } else if(loc[0]=='fr'){
            l ="/fr/"
        } else if(loc[0]=='de'){
            l ="/"
        }
        var t = parseConf.urlcheckURL.replace(/\/en\//, l)
        Qt.openUrlExternally(t+url)
    }

    function urlRankView(url){
        Qt.openUrlExternally(parseConf.rankURL+url)
    }

    function getWayBackUrl(url){
        var backUrl=JS.getNakedUrl(url.toString())
        return WebArchive.url+"*/"+JS.getNonMobileUrl(backUrl)
    }
    function wayBackView(url){
        Qt.openUrlExternally(getWayBackUrl(url))
    }
    function getWayBackDate(url){
        var currentUrl = url.toString()
        console.log(currentUrl)
        var regex = new RegExp(WebArchive.regex)
        var match = currentUrl.match(regex)
        var date=""
        if(match){
           date=match[WebArchive.matchDate]
       }
       return date
    }

    function getWayBackPreFix(url){
        var date=getWayBackDate(url)
        var preFix=""
        if(date.length>1){
           date = date.substring(0, 8)+"000000"
           preFix=WebArchive.url+date+"/"
       }
       return preFix
    }
    function getUrl(url){
       var out=url.toString()
        var regex = new RegExp(WebArchive.regex)
        var match = out.match(regex)
        if(match){
           out="http"+match[WebArchive.matchUrl]
       }
       return out
    }
/*
//  http://web.archive.org/web/20160122220340/https://en.wikipedia.org/wiki/Main_Page
*/


//
// https://www.mediawiki.org/wiki/User:Deo_Favente/Palette/OO
// 
    function websiteSafeCode(safe, updatedAt){
        var out={color:"darkGrey",icon:undefined};
        if(safe==0){
            out.color="#94BD5E"
            out.icon="face_ok";
        } else if(safe==1){
            out.color="#E6E64C"
            out.icon="face";
        } else if(safe>1){
            out.color="#FF3333"
            out.icon="face_ko";
        }
        return out;
    }

    function updateWebsite(website){
         var body = '{"safe":'+website.safe
         if(parseConf.rank2Loc1>0){
             body += ',"rank":"'+website.rank+'"}'
         } else {
             body += '}'
         }
         doRequest("PUT", '/WebSite/'+website.objectId, body)
//         doRequestCallback(websiteUpdated, "PUT", '/WebSite/'+website.objectId, body)
    }

    function updateSecondaryPatterns(){
        var sites=DB.loadWebSites().rows
        var unique=[]
        for (var i=0; i<sites.length; i++) {
            if(sites[i].secondaryPatterns){
                unique = JS.arrayUnique(unique.concat(JSON.parse(sites[i].secondaryPatterns)));
            }
        }
        secondaryPatterns = unique
        patternsUpdated()
    }
}
