import QtQuick 2.4
import Ubuntu.Components 1.3

import "databasemodule_v2.js" as DB
import "util.js" as JS

ListItem {
    id: webSiteComponent

    height: columnContent.height

    property bool isExpanded: false
    property string webSiteTitle
    property int webSiteId
    property int modelIndex
    property real webSitePosition

    onClicked: isExpanded = !isExpanded ;
    onPressAndHold: {
        webSiteManagement.isReordined=false
        pageStack.push(Qt.resolvedUrl("ReorderWebSites.qml"))
    }

    onStateChanged:{
        if (isExpanded){
            reloadWebSite()
        }
    }

    leadingActions: ListItemActions {
        actions: [
            Action {
                iconName: "delete"
                onTriggered: {
                    console.log("item about to be removed: ", webSiteComponent.webSiteId)
                    var result = DB.deleteWebSite(webSiteComponent.webSiteId, webSiteComponent.webSitePosition)
                    if (result.rowsAffected == 1)
                    {
                        webSiteManagement.removeModelItem(webSiteComponent.modelIndex)
                    }
                    else
                    {
                        webSiteManagement.reloadWebSites()
                    }
                    util.updateSecondaryPatterns()
                }
            }
        ]
    }
    trailingActions: ListItemActions {}

    Behavior on height {
        NumberAnimation { duration: 150; easing.type: Easing.OutQuad }
    }

    function reloadWebSite (){
        detailModel.clear()
        var w =  DB.getWebSite(webSiteId)

        detailModel.append(
        {
            "id": 1
            ,"title" : w.rows[0].source
        })
        detailModel.append(
        {
            "id": 2
            ,"title" : w.rows[0].pattern ? w.rows[0].pattern : w.rows[0].insertTime
        })
        detailModel.append(
        {
            "id": 3
            ,"title" : w.rows[0].pattern ? w.rows[0].home : ''
        })
    }

    Column{
        id: columnContent
        anchors{ left: parent.left; right: parent.right }

        /*
          webSite item
        */
        Item
        {
            anchors{ left: parent.left; right: parent.right }
            height: units.gu(5)

            Row{
                id: rowWebSiteContent
                anchors
                {
                    top: parent.top; bottom: parent.bottom; left: parent.left;
                    leftMargin: units.gu(1); topMargin: units.gu(0.7); bottomMargin: units.gu(1);
                }
                spacing: units.gu(2)
                Label{
                    id: labelWebSiteTitle
                    objectName: "labelWebSiteTitle"
                    anchors.verticalCenter: parent.verticalCenter
                    text: JS.bookmarkId(webSitePosition) ? '>>> '+webSiteTitle : webSiteTitle
                }
            }

            Image{
                id: imgArrow
                anchors
                {
                    right: parent.right; top: parent.top; bottom: parent.bottom;
                    topMargin: units.gu(1.5); bottomMargin: units.gu(1.5); rightMargin: units.gu(2)
                }
                fillMode: Image.PreserveAspectFit
                source: Qt.resolvedUrl("../icons/go-to.svg")
                smooth: true

                Behavior on rotation { UbuntuNumberAnimation{} }

                states: [
                    State {
                        name: "expanded"
                        when: webSiteComponent.isExpanded

                        PropertyChanges
                        {
                            target: imgArrow
                            rotation: 90
                        }
                    }
                ]
            }

        }

        /*
          details listview
        */
        ListView
        {
            id: detailList
            anchors{ left: parent.left; right: parent.right }
            height: 0
            opacity: 0
            interactive: false
            clip: true

            property int transitionDuration: 250

            Behavior on opacity {
                PropertyAnimation { duration: 150; easing.type: Easing.OutQuad }
            }

            model: ListModel {
                id: detailModel
            }
            delegate: Item {
                id: delegateDetail
                width: detailList.width
                height: detailItem.height

                property int webSiteId: webSiteComponent.webSiteId
                property int modelIndex: webSiteComponent.modelIndex

                DetailComponent {
                    id: detailItem
                    text: model.title
                    webSiteId: delegateDetail.webSiteId
                    modelIndex: delegateDetail.modelIndex
                    width: webSiteList.width
                    height: units.gu(6)
                }
            }
        }
    }

    states: [
        State {
            name: "expanded"
            when: isExpanded
            PropertyChanges
            {
                target: detailList
                height: detailList.contentItem.childrenRect.height
                opacity: 1
            }
        },

        State {
            name: ""
//            when: isExpanded
            PropertyChanges
            {
                target: detailList
                height: 0
                opacity: 0
            }
        }
    ]
    onContentMovementStarted: {
        isExpanded = false
    }

    Connections {
        target: editWebSite

        onWebSiteEdited:{
            if (isExpanded){
                reloadWebSite()
            }
        }
    }
}
