import QtQuick 2.4
import Ubuntu.Components 1.3
import Ubuntu.Components.Popups 1.3
// import Ubuntu.Layouts 1.0

import "databasemodule_v2.js" as DB

Page {
    id: webSiteManagement
    objectName: "webSiteManagement"

    title: i18n.tr("Setup")
    flickable: content
//    tools: toolbar
    property bool isReordined: false
    head.actions: [settingsAction, helpAction, refreshAction, resetAction]

    onStateChanged:{
        if (isReordined){
            reloadWebSites ()
        }
    }

//    Component.onCompleted: {
// commented because cannot be done before db ubgrade!
//        reloadWebSites ()
//    }

    function reloadWebSites ()
    {
        webSiteModel.clear()
        var webSites = DB.loadWebSites()
        for (var i=0; i < webSites.rows.length; i++){
            webSiteModel.append(
            {
                "id": webSites.rows[i].id
                ,"title": webSites.rows[i].title
                ,"position": webSites.rows[i].position
            })
        }
    }

    function removeModelItem (modelIndex){
        webSiteModel.remove(modelIndex)
        console.log("removeWebSiteItem ", modelIndex)
    }

    /*
      main content
    */
    Flickable {
        id: content
        anchors {
            left: parent.left
            right: parent.right
            top: parent.top
            bottom: fakePanel.top
        }
        contentHeight: contentItem.childrenRect.height
 
        Column{
            anchors{ left: parent.left; right: parent.right }

            /*
              WebSite listview
            */
            ListView {
                id: webSiteList
                anchors{ left: parent.left; right: parent.right }
                height: contentItem.childrenRect.height
                interactive: false
                currentIndex: -1

                model: ListModel {
                    id: webSiteModel
                }
                delegate: MouseArea {
                    id: delegateRoot
                    anchors{ left: parent.left; right: parent.right }
                    height: webSiteItem.height
                    WebSiteComponent{
                        id: webSiteItem
                        webSitePosition: model.position
                        webSiteTitle: model.title
                        webSiteId: model.id
                        modelIndex: index
                    }
                }
            } // listview
        } // column
    } // flickable
    Rectangle {
        id: fakePanel

        color: "#fafafa"
        anchors {
            left: parent.left
            right: parent.right
            bottom: parent.bottom
        }
        height: units.gu(6)
        Item {
            anchors.fill: parent

            Button {
                id: actionsButton
                text: i18n.tr("Add     ")
                anchors {
                    verticalCenter: parent.verticalCenter
                    right: parent.right
                    rightMargin: units.gu(1)
                }
                onClicked: pageStack.push(addWebSitePage /*, {"isDirty" : true}*/)
            }
            Button {
                id: downloadButton
                text: i18n.tr("Download")
                anchors {
                    verticalCenter: parent.verticalCenter
                    left: parent.left
                    leftMargin: units.gu(1)
                }
                onClicked: pageStack.push(downloadWebSitePage)
            }

        }
    } // Rectangle fakePanel

/*-------------------------- Actions ---------------------------- */
    Action {
        id: refreshAction

        text:  i18n.tr("Refresh")
        iconSource: Qt.resolvedUrl("../icons/reload.svg")
        onTriggered: reloadWebSites()
    }
    Action {
        id: helpAction

        text:  i18n.tr("Info")
        iconSource: Qt.resolvedUrl("../icons/info.svg")
        onTriggered: {
            log.statistics('info')
            pageStack.push(infoPage)
        }
    }
    Action {
        id: settingsAction

        text:  i18n.tr("Settings")
        iconSource: Qt.resolvedUrl("../icons/settings.svg")
        onTriggered: pageStack.push(settingsPage)
    }
    Action {
        id: resetAction

        text:  i18n.tr("Reset")
        iconSource: Qt.resolvedUrl("../icons/reset.svg")
        onTriggered: PopupUtils.open(dialog)
    }
    /* -------------------------- dialogue ---------------------------- */
    Item {
        width: units.gu(80)
        height: units.gu(80)
        Component {
             id: dialog
             Dialog {
                 id: dialogue
                 title: i18n.tr("Reset Websites")
                 text: i18n.tr("Are you sure to reset Websites?")
                 Button {
                     text: i18n.tr("reset")
                     color: UbuntuColors.green
                     onClicked: {
                         log.message(log.warn, "resetWebSites")
                         log.statistics('resetdb')
                         parseRest.getDefWebSites(mainView.resetWebSites)
                         PopupUtils.close(dialogue)
                     }
                 }
                 Button {
                     text: i18n.tr("cancel")
                     onClicked: PopupUtils.close(dialogue)
                 }
            }
        }
    }
    states: [
        State {
            name: "reordined"
            when: isReordined
        },

        State {
            name: ""
//            when: isReordined
        }
    ]
    /* -------------------------- connections ---------------------------- */
    Connections {
        target: addWebSitePage
        onWebSiteAdded:{
            reloadWebSites ()
        }
    }
    Connections {
        target: editWebSite
        onWebSiteEdited:{
             webSiteModel.setProperty(id, "title", label)
        }
    }
    Connections {
        target: mainView
        onWebSitesReset:{
            reloadWebSites ()
        }
    }
}
