/*
 * Copyright 2013-2015 Canonical Ltd.
 *
 * This file is part of webbrowser-app.
 *
 * webbrowser-app is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * webbrowser-app is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import QtQuick 2.4
import Ubuntu.Components 1.3
import Ubuntu.Components.Popups 1.3
import Ubuntu.Web 0.2
// import webbrowsercommon.private 0.1
import "actions" as Actions
import com.canonical.Oxide 1.8 as Oxide

WebView {
    id: webview

    property var currentWebview: webview

    /*experimental.certificateVerificationDialog: CertificateVerificationDialog {}
    experimental.proxyAuthenticationDialog: ProxyAuthenticationDialog {}
// mrn 13/7/2015 commented four lines below!
    alertDialog: AlertDialog {}
    confirmDialog: ConfirmDialog {}
    promptDialog: PromptDialog {}
    beforeUnloadDialog: BeforeUnloadDialog {}*/
    filePicker: contentPickerLoader.item || filePickerLoader.item
    QtObject {
        id: internal

        readonly property var downloadMimeTypesBlacklist: [
            "application/x-shockwave-flash", // http://launchpad.net/bugs/1379806
        ]
        function instantiateShareComponent() {
            var component = Qt.createComponent("Share.qml")
            if (component.status == Component.Ready) {
                var share = component.createObject(webview)
                share.onDone.connect(share.destroy)
                return share
            }
            return null
        }

        function shareLink(url, title) {
            var share = instantiateShareComponent()
            if (share) share.shareLink(url, title)
        }

        function shareText(text) {
            var share = instantiateShareComponent()
            if (share) share.shareText(text)
        }

    }

    onFullscreenRequested: webview.fullscreen = fullscreen

    onDownloadRequested: {
        if (!request.suggestedFilename && request.mimeType &&
            internal.downloadMimeTypesBlacklist.indexOf(request.mimeType) > -1) {
            return
        }

        if (downloadLoader.status == Loader.Ready) {
            var headers = { }
            if (request.cookies.length > 0) {
                headers["Cookie"] = request.cookies.join(";")
            }
            if (request.referrer) {
                headers["Referer"] = request.referrer
            }
            headers["User-Agent"] = webview.context.userAgent
            // Work around https://launchpad.net/bugs/1487090 by guessing the mime type
            // from the suggested filename or URL if oxide hasn’t provided one.
            var mimeType = request.mimeType
            if (!mimeType) {
                mimeType = MimeDatabase.filenameToMimeType(request.suggestedFilename)
            }
            if (!mimeType) {
                var scheme = request.url.toString().split('://').shift().toLowerCase()
                var filename = request.url.toString().split('/').pop().split('?').shift()
                if ((scheme == "file") || (filename.indexOf('.') > -1)) {
                    mimeType = MimeDatabase.filenameToMimeType(filename)
                }
            }
            downloadLoader.item.downloadMimeType(request.url, mimeType, headers, request.suggestedFilename)
        } else {
            // Desktop form factor case
            Qt.openUrlExternally(request.url)
        }
    }

    onHttpAuthenticationRequested: {
        PopupUtils.open(Qt.resolvedUrl("HttpAuthenticationDialog.qml"),
                        webview.currentWebview, {"request": request})
    }

    Loader {
        id: contentPickerLoader
        source: "ContentPickerDialog.qml"
        asynchronous: true
    }
    Loader {
        id: filePickerLoader
        source: "FilePickerDialog.qml"
        asynchronous: true
    }

    Loader {
        id: downloadLoader
        // TODO: Use the ubuntu download manager on desktop as well
        //  (https://launchpad.net/bugs/1477310). This will require to have
        //  ubuntu-download-manager in main (https://launchpad.net/bugs/1488425).
        source: "Downloader.qml"
        asynchronous: true
    }
// mrn 14/7/2015: added contextualActions (WebViewImplOxide.qml)
// mrn 5/12/2015: replaced with a new list
    contextualActions: ActionList {
        Actions.CopyLink {
            enabled: webview.contextModel && webview.contextModel.linkUrl.toString()
            onTriggered: Clipboard.push(["text/plain", contextModel.linkUrl.toString()])
        }
        Actions.CopyImage {
            enabled: webview.contextModel &&
                     (webview.contextModel.mediaType === Oxide.WebView.MediaTypeImage) &&
                     webview.contextModel.srcUrl.toString()
            onTriggered: Clipboard.push(["text/plain", contextModel.srcUrl.toString()])
        }
        Actions.Undo {
            enabled: webview.contextModel && webview.contextModel.isEditable &&
                     (webview.contextModel.editFlags & Oxide.WebView.UndoCapability)
            onTriggered: webview.executeEditingCommand(Oxide.WebView.EditingCommandUndo)
        }
        Actions.Redo {
            enabled: webview.contextModel && webview.contextModel.isEditable &&
                     (webview.contextModel.editFlags & Oxide.WebView.RedoCapability)
            onTriggered: webview.executeEditingCommand(Oxide.WebView.EditingCommandRedo)
        }
        Actions.Cut {
            enabled: webview.contextModel && webview.contextModel.isEditable &&
                     (webview.contextModel.editFlags & Oxide.WebView.CutCapability)
            onTriggered: webview.executeEditingCommand(Oxide.WebView.EditingCommandCut)
        }
        Actions.Copy {
            enabled: webview.contextModel && webview.contextModel.isEditable &&
                     (webview.contextModel.editFlags & Oxide.WebView.CopyCapability)
            onTriggered: webview.executeEditingCommand(Oxide.WebView.EditingCommandCopy)
        }
        Actions.Paste {
            enabled: webview.contextModel && webview.contextModel.isEditable &&
                     (webview.contextModel.editFlags & Oxide.WebView.PasteCapability)
            onTriggered: webview.executeEditingCommand(Oxide.WebView.EditingCommandPaste)
        }
        Actions.Erase {
            enabled: webview.contextModel && webview.contextModel.isEditable &&
                     (webview.contextModel.editFlags & Oxide.WebView.EraseCapability)
            onTriggered: webview.executeEditingCommand(Oxide.WebView.EditingCommandErase)
        }
        Actions.SelectAll {
            enabled: webview.contextModel && webview.contextModel.isEditable &&
                     (webview.contextModel.editFlags & Oxide.WebView.SelectAllCapability)
            onTriggered: webview.executeEditingCommand(Oxide.WebView.EditingCommandSelectAll)
        }
        // mrn 12/12/2015: from Browser.qml...
        Actions.Share {
            objectName: "ShareContextualAction"
            enabled: webview.contextModel &&
                     (webview.contextModel.linkUrl.toString() || webview.contextModel.selectionText)
            onTriggered: {
                if (webview.contextModel.linkUrl.toString()) {
                    internal.shareLink(contextModel.linkUrl.toString(), webview.contextModel.linkText)
                } else if (webview.contextModel.selectionText) {
                    internal.shareText(webview.contextModel.selectionText)
                }
            }
        }
    }
}
