import QtQuick 2.4
import Ubuntu.Components 1.3
import Ubuntu.Web 0.2
import com.canonical.Oxide 1.8 as Oxide
import Ubuntu.Content 1.1
// import Ubuntu.Components.Popups 1.3
import "databasemodule_v2.js" as DB
import "util.js" as JS


WebViewImpl {
    id: webview

	url: myUrl
    property string localUserAgentOverride: ""

	preferences.localStorageEnabled: true
	preferences.allowFileAccessFromFileUrls: true
	preferences.allowUniversalAccessFromFileUrls: true
	preferences.appCacheEnabled: true
	preferences.javascriptCanAccessClipboard: true

    property var popupController
    property var overlayViewsParent: webview.parent

	anchors {
	    fill: parent
	    bottom: parent.bottom
	} 
	width: parent.width 
	height: parent.height

    currentWebview: webview
    context: WebContext {
        id: webcontext
        userAgent: localUserAgentOverride
    }

    onNewViewRequested: popupController.createPopupView(overlayViewsParent, request, true, context)

    function isValid (url){ 
        var sites = DB.loadWebSites()
        for (var i=0; i<sites.rows.length; i++) {
            if (JS.doesMatch(url, sites.rows[i].pattern) == true) {
                return true;
            }
        }
        if(settingsPage.secondaryPatterns){
            for (var i=0; i<settingsPage.secondaryPatterns.length; i++) {
                if (JS.doesMatch(url, settingsPage.secondaryPatterns[i]) == true) {
                    return true;
                }
            }
        }
        return false; 
    }

    function isNewForegroundWebViewDisposition(disposition) {
        return disposition === Oxide.NavigationRequest.DispositionNewPopup ||
               disposition === Oxide.NavigationRequest.DispositionNewForegroundTab;
    }

    function navigationRequestedDelegate(request) {
        var url = request.url.toString();
        if (isNewForegroundWebViewDisposition(request.disposition) && optionsKeeper.allowPopup()) {
            request.action = Oxide.NavigationRequest.ActionAccept
            popupController.handleNewForegroundNavigationRequest(url, request, true)
            return
        }
        if(isValid(url) == false) {
            console.warn("Opening remote: " + url);
            Qt.openUrlExternally(url)
            request.action = Oxide.NavigationRequest.ActionReject
        }
    }

    Component.onCompleted: {
	    preferences.localStorageEnabled = true
	    if(Qt.application.arguments[2] != undefined && isValid(Qt.application.arguments[1]) == true) {
	        url = Qt.application.arguments[1]
	    }
	    console.warn("url is: >>>>" + url)
	}

	onGeolocationPermissionRequested: { request.accept() }
}

