.pragma library // I hope this will prevent the waste of memory.
.import QtQuick.LocalStorage 2.0 as SQL
.import "../config.js" as Conf
.import "util.js" as JS

/* For internal usage in module.
 */
var gDbCache = undefined
function openStdDataBase() {
    if (gDbCache === undefined) {
        gDbCache = SQL.LocalStorage.openDatabaseSync(Conf.webappName, "1.0", "App main DB", 100000)
    }

    // check table exist after open the database
    gDbCache.transaction(function(tx) {
        checkTableExists(tx/*, opts*/);
    })
    return gDbCache
}

// Keep in mind that VARCHAR(x) converted to TEXT in SQLITE.
function checkTableExists(transaction /* and additional string keys */) {
//    transaction.executeSql('PRAGMA foreign_keys = ON;')   // enable foreign key support
    transaction.executeSql("CREATE TABLE IF NOT EXISTS web_site  (id  INTEGER  NOT NULL PRIMARY KEY AUTOINCREMENT,source  TEXT  NULL,title  TEXT  NULL,pattern  TEXT  NULL, home TEXT NULL, position REAL NULL, secondaryPatterns TEXT NULL, insertTime TEXT NULL);")
    transaction.executeSql("CREATE TABLE IF NOT EXISTS bmt_rules  (id  INTEGER  NOT NULL PRIMARY KEY AUTOINCREMENT, pattern  TEXT  NULL, regexSrc TEXT NULL, regexDest TEXT NULL, respTitle INTEGER NULL, useParams INTEGER NULL, useFragment INTEGER NULL, lat REAL NULL, long REAL NULL, zoom INTEGER NULL, type INTEGER NULL);")
}

/* update database scheme function
 * Pls be careful..
 */
function updateDatabase(currentDBVersion) {
    var cdbv = currentDBVersion
    switch (cdbv) {
    case 1.1:
    case 1.2:
        // popolate optionskeeper new fields 
        break
    case 1.3:
        var db = openStdDataBase()
        var dbResult
        db.transaction(function(tx) {
            dbResult = tx.executeSql("alter table web_site add home text")
            console.log("database updated: ", JSON.stringify(dbResult))
        }
        )
        break
    case 1.4:
        var db = openStdDataBase()
        var dbResult
        db.transaction(function(tx) {
            dbResult = tx.executeSql("alter table web_site add position real")
            console.log("database updated: ", JSON.stringify(dbResult))
            dbResult = tx.executeSql("alter table web_site add secondaryPatterns text")
            console.log("database updated: ", JSON.stringify(dbResult))
        }
        )
        break
    case 1.5:
        var db = openStdDataBase()
        var dbResult
        db.transaction(function(tx) {
            dbResult = tx.executeSql("alter table web_site add insertTime text")
            console.log("database updated: ", JSON.stringify(dbResult))
        }
        )
        break
/*
    case 1.6:
        // already newest
        // do nothing
        return false
*/
    default:
        return false // do nothing
    }
    return true
}

/* web_site operations
 * include select, insert, update and delete operations
 */
function loadWebSites()
{
    var db = openStdDataBase()
    var dbResult
    db.transaction(function(tx) {
        dbResult = tx.executeSql("SELECT * FROM web_site ORDER BY position")
        console.log("web_site SELECTED: ", dbResult.rows.length)
    }
    )
    return dbResult;  // I suggest that return the whole result in order to know if error occurs
}

function getWebSite(id)
{
    var db = openStdDataBase()
    var dbResult
    db.transaction(function(tx) {
        dbResult = tx.executeSql("SELECT * FROM web_site WHERE id=?", [id])
    }
    )
    return dbResult;  
}

function webSitesCount() {
    var db = openStdDataBase()
    var dbResult

    db.transaction(function(tx) {
        dbResult = tx.executeSql("SELECT Count(*) FROM web_site")
        var obj = dbResult.rows.item(0)
        dbResult.count = obj["Count(*)"]
    }
    )
    return dbResult
}

function addBookmark(title, source, position){
    return addWebSite(title, source, null, null, position)
}

// insert
function addWebSite(title, source, pattern, secondaryPatterns, position)  // from user input
{
    var dbResult
    var db = openStdDataBase()
    db.transaction(function (tx) {
        /* Check uniqueness.
         */
        dbResult = tx.executeSql("SELECT * FROM web_site WHERE title=? OR source=?", [title, source])
        if (dbResult.rows.length > 0) {
            console.log("Database, addWebSite: web_site already exists title: ", title, "url", source, "ID", dbResult.rows.item(0).id)
            dbResult = {"error": true, "exist": true}
            return
        }
        /* Check max count.
         */
        dbResult = tx.executeSql("SELECT Count(*) FROM web_site")
        var obj = dbResult.rows.item(0)
        var count = obj["Count(*)"]
        if( count > 19 ){
            console.log("Database, addWebSite: number of max web sites has been exceeded : ", count) 
            dbResult = {"error": true}
            return
        }
        if (!position){
            dbResult = tx.executeSql("SELECT Max(position) FROM web_site")
            var obj = dbResult.rows.item(0)
            var max = obj["Max(position)"]
            position = JS.positionId(max) + 1
            console.log("new position: "+position)
        }
        var d = new Date();
        dbResult = tx.executeSql('INSERT INTO web_site '
                 +'(title, source, pattern, secondaryPatterns, home, position, insertTime)'                   
                 +' VALUES(?, ?, ?, ?, ?, ?, ?)',
                 [title , source, pattern, secondaryPatterns, source, position, d.toLocaleString()])
        console.log("web_site INSERT ID: ", dbResult.insertId)

        dbResult.webSiteId = tx.executeSql("SELECT * FROM web_site WHERE title=?", [title]).rows.item(0).id
        console.log("dbResult.webSiteId", dbResult.webSiteId)
    }
    )
    return dbResult;
}
// update
function updateWebSiteSecondaryPatterns(id, patterns)  // from user input
{
    var db = openStdDataBase()
    var dbResult
    db.transaction(function (tx) {
        dbResult = tx.executeSql('UPDATE web_site SET secondaryPatterns=? WHERE id=?',
                                 [patterns, id])
        console.log("web_site updateWebSiteHome, AFFECTED ROWS: ", dbResult.rowsAffected)
    }
    )
    return dbResult
}
function updateWebSitePosition(id, position)  // from user input
{
    var db = openStdDataBase()
    var dbResult
    db.transaction(function (tx) {
        dbResult = tx.executeSql('UPDATE web_site SET position=? WHERE id=?',
                                 [position, id])
        console.log("web_site updateWebSiteHome, AFFECTED ROWS: ", dbResult.rowsAffected)
    }
    )
    return dbResult
}
function updateWebSiteHome(id, home)  // from user input
{
    var db = openStdDataBase()
    var dbResult
    db.transaction(function (tx) {
        dbResult = tx.executeSql('UPDATE web_site SET home=? WHERE id=?',
                                 [home, id])
        console.log("web_site updateWebSiteHome, AFFECTED ROWS: ", dbResult.rowsAffected)
    }
    )
    return dbResult
}
// reset
function resetWebSiteHome(id)  // from user input
{
    var db = openStdDataBase()
    var dbResult
    db.transaction(function (tx) {
        dbResult = tx.executeSql('UPDATE web_site SET home=source WHERE id=?',
                                 [id])
        console.log("web_site resetWebSiteHome, AFFECTED ROWS: ", dbResult.rowsAffected)
    }
    )
    return dbResult
}

// update
function updateWebSiteByUser(id, title, source, pattern)  // from user input
{
    var db = openStdDataBase()
    var dbResult
    db.transaction(function (tx) {
        /* Check uniqueness.
         */
        dbResult = tx.executeSql("SELECT * FROM web_site WHERE title=? AND id<>?", [title, id])
        if (dbResult.rows.length > 0) {
            console.log("Database, updateWebSite: already exist web_site with title: ", title, "ID", dbResult.rows.item(0).id)
            dbResult = {"error": true, "exist": true}
            return
        }
        dbResult = tx.executeSql('UPDATE web_site SET title=?, source=?, pattern=? WHERE id=?',
                                 [title, source, pattern, id])
        console.log("web_site updateWebSiteByUser, AFFECTED ROWS: ", dbResult.rowsAffected)
    }
    )
    return dbResult
}
function updateWebSiteTitle(id, title)  // from user input
{
    var db = openStdDataBase()
    var dbResult
    db.transaction(function (tx) {
        /* Check uniqueness.
         */
        dbResult = tx.executeSql("SELECT * FROM web_site WHERE title=? AND id<>?", [title, id])
        if (dbResult.rows.length > 0) {
            console.log("Database, updateWebSite: already exist web_site with title: ", title, "ID", dbResult.rows.item(0).id)
            dbResult = {"error": true, "exist": true}
            return
        }
        dbResult = tx.executeSql('UPDATE web_site SET title=? WHERE id=?',
                                 [title, id])
        console.log("web_site updateWebSiteByUser, AFFECTED ROWS: ", dbResult.rowsAffected)
    }
    )
    return dbResult
}

// delete
function deleteWebSite(id, position)
{
    var db = openStdDataBase()
    var dbResult
    db.transaction(function (tx) {
//        tx.executeSql('PRAGMA foreign_keys = ON;')   // enable foreign key support
        if(!position || JS.bookmarkId(position)){
            dbResult = tx.executeSql('delete from web_site WHERE id=?',
                                 [id])
        } else {
            var pos = JS.positionId(position)
            dbResult = tx.executeSql('delete from web_site WHERE position>? AND position<?',
                                 [pos-.1, pos+.9])
        }
        console.log("web_site delete, AFFECTED ROWS: ", dbResult.rowsAffected)
    }
    )
    return dbResult
}

/* bmt_rules operations
 * include select, insert, update and delete operations
 */
function loadBmtRules()
{
    var db = openStdDataBase()
    var dbResult
    db.transaction(function(tx) {
        dbResult = tx.executeSql("SELECT * FROM bmt_rules ORDER BY id")
        console.log("bmt_rules SELECTED: ", dbResult.rows.length)
    }
    )
    return dbResult;  
}

// insert
function addBmtRules(rule)  // from user input
{
    var dbResult
    var db = openStdDataBase()
    db.transaction(function (tx) {
        dbResult = tx.executeSql('INSERT INTO bmt_rules '
                    +'(pattern, regexSrc, regexDest, respTitle, useParams, useFragment, lat, long, zoom, type) '
                    +'VALUES(?, ?, ?, ?, ?, ?, ?, ?, ?, ?)',
                    [rule.pattern, rule.regexSrc, rule.regexDest, rule.respTitle, rule.useParams, 
                     rule.useFragment, rule.lat, rule.long, rule.zoom, rule.type])
        console.log("bmt_rules INSERT ID: ", dbResult.insertId)
    }
    )
    return dbResult;
}


/* operations for testing
 * include clear and drop operations
 * not completed yet
 *
 */
// clear
function clearData(table)
{
    var db = openStdDataBase()

    switch(table)
    {
    case "web_site":
        db.transaction(function(tx) {
            tx.executeSql("delete from web_site")
            console.log("web_site clear")
        }
        )
        break;
    case "bmt_rules":
        db.transaction(function(tx) {
            tx.executeSql("delete from bmt_rules")
            console.log("bmt_rules clear")
        }
        )
        break;
    default:
        db.transaction(function(tx) {
            tx.executeSql("delete from web_site")
            tx.executeSql("delete from bmt_rules")
            console.log("DATABASE clear")
        }
        )
    }
}

// drop
function dropTable(table)
{
    var db = openStdDataBase()

    switch(table)
    {
    case "web_site":
        db.transaction(function(tx) {
            tx.executeSql("DROP TABLE IF EXISTS web_site")
            console.log("web_site deleted")
        }
        )
        break;
    case "bmt_rules":
        db.transaction(function(tx) {
            tx.executeSql("DROP TABLE IF EXISTS bmt_rules")
            console.log("bmt_rules deleted")
        }
        )
        break;
    default:
        db.transaction(function(tx) {
            tx.executeSql("DROP TABLE IF EXISTS web_site")
            tx.executeSql("DROP TABLE IF EXISTS bmt_rules")
            console.log("DATABASE deleted")
        }
        )
    }
}
